__author__ = 'Michele Bannister'

import math

import matplotlib.pyplot as plt
from matplotlib import rcParams
from matplotlib.ticker import MultipleLocator
import numpy as np
import ephem

import brewer2mpl
import parsers
import parameters
import plot_fanciness

set2 = brewer2mpl.get_map('Set2', 'qualitative', 8).mpl_colors
rcParams['font.size'] = 12  # good for posters/slides
rcParams['patch.facecolor'] = set2[0]


def top_down_SolarSystem(discoveries,
                         extent=68,
                         plot_blocks=True,
                         future_blocks=True,
                         plot_Ijiraq=True,
                         label_blocks=True):
    """
    Plot the OSSOS discoveries on a top-down Solar System showing the position of Neptune and model TNOs.
    Discoveries are plotted each at their time of discovery according to the value in the Version Release.
    Coordinates should be polar to account for RA hours, radial axis is in AU.
    :return: a saved plot

    This is not a 'projection' of the particles into any
    common plane.  Each wedge is a different latitude above
    the plane, but inside each wedge it is heliocentric distance
    vs RA.
     That is fine.  It's just important to know that if someone
    asks we know this is NOT (for example) a projection of each
    object down into the ecliptic.
     A very minor future long-term improvement is that the galactic
    plane wedge is not (I don't think) symmetric around 18h and 6h.
    I think that lines of constant (say 15 degree galactic lat) are
    centered a little off (I think the wedge would 'twist' a little
    bit counter-clockwise).  I am not absolutely sure along the
    ecliptic where the b= +/- 15 degree lines are but I don't
    think they are symmetric?
    """
    fig = plt.figure(figsize=(6, 6))
    rect = [0.1, 0.1, 0.8, .8]  # the plot occupies not all the figspace

    ax1 = fig.add_axes(rect, polar=True, frameon=False)  # theta (RA) is zero at E, increases anticlockwise
    ax1.set_aspect('equal')

    ax1.set_rlim(0, extent)
    ax1.set_rgrids([20, 40, 60], labels=["", "", '20 AU', '40 AU', '60 AU'], angle=197, alpha=0.45)  # angle = 308
    ax1.yaxis.set_major_locator(MultipleLocator(20))
    ax1.xaxis.set_major_locator(MultipleLocator(math.radians(15)))  # every 2 hours
    ax1.grid(axis='x', color='k', linestyle='--', alpha=0.2)
    ax1.set_xticklabels(['', '0h', "", '2h', "", '4h', "", '6h', "", '8h', "", '10h', "", '12h', "", '14h', "", '16h',
                         "", '18h', "", '20h', "", '22h', "", ],
                        # ""])  # str(r)+'h' for r in range(-1,24)],
                        #        ['', '0h', '2h', '4h', '6h', '8h', '10h', '12h', '14h', '16h', '18h', '20h', '22h'],
                        color='b', alpha=0.6)  # otherwise they get in the way


    # plot exclusion zones due to Galactic plane: RAs indicate where bar starts, rather than its centre angle
    # can I do this with a warp, or will it make more sense to just plot stellar density? maybe that?
    width = math.radians(3 * 15)
    # plt.bar(math.radians(4.5 * 15), extent, width=width, color=plot_fanciness.ALMOST_BLACK, linewidth=0, alpha=0.2)
    # plt.bar(math.radians(16.5 * 15), extent, width=width, color=plot_fanciness.ALMOST_BLACK, linewidth=0, alpha=0.2)
    # ax1.annotate('galactic plane', (math.radians(6.9 * 15), extent - 15), size=10, color='k', alpha=0.45)
    # ax1.annotate('  galactic plane\navoidance zone', (math.radians(16.9 * 15), extent - 12), size=10, color='k', alpha=0.45)


    # FIXME: should probably convert hours to ecliptic coords with a bit more finesse than just overplotting it
    # truncate these at 8 AU to show that we don't have sensitivity in close; detect Ijiraq at 9.80 AU
    # extend to 75 AU as indicative only, sensitivity is to ~300 AU
    # again RAs indicate where bar starts, so subtract half the block width from the block centrepoints
    # and approximate blocks as math.radians(7) degrees wide.

    for blockname, block in parameters.BLOCKS.items():  # ["14:15:28.89", "15:58:01.35", "00:54:00.00", "01:30:00.00"]:
        if plot_blocks:
            if blockname.startswith('13') or blockname.startswith('14'):
                colour = 'b'
                alpha = 0.1
                cmap = plt.get_cmap('Blues')  # colorbrewer.sequential.Blues_4.mpl_colors
                # if blockname.endswith('AO'):
                #     colour = '#E47833'
                #     alpha = 0.17
                #     cmap = plt.get_cmap('Oranges')  # colorbrewer.sequential.Oranges_3.mpl_colors

                # want to apply a colour gradient to the block
                # luminance = [[.6, .6],[.7,.7]]
                # gradient = np.linspace(0, 1, 256)
                # gradient = np.vstack((gradient, gradient))
                #
                # r_i = np.linspace(8, extent, 256)
                # r_i = np.vstack((r_i, r_i))
                # theta_i = np.linspace(ephem.hours(block["RA"]) - math.radians(3.5),
                #                       ephem.hours(block["RA"]) + math.radians(3.5), 7)
                # grid = np.meshgrid(theta_i, r_i)
                #
                # ax1.imshow(r_i,
                #            extent=(ephem.hours(block["RA"]) - math.radians(3.5),
                #                    ephem.hours(block["RA"]) + math.radians(3.5),
                #                    8, extent),
                #            cmap=cmap)

                bar = ax1.bar(ephem.hours(block["RA"]) - math.radians(3.5), extent, linewidth=0.1,
                              width=math.radians(7), bottom=8, zorder=0, color=colour, alpha=alpha)
                # bar[0].set_facecolor(cmap)
                # bar[0].set_alpha(0.8)
                if label_blocks:
                    ax1.annotate(blockname[3], (ephem.hours(block["RA"]) + math.radians(0.3), extent + 0.15), size=15,
                                 color='b')

        if future_blocks:
            if blockname.startswith('15'):
                plt.bar(ephem.hours(block["RA"]) - math.radians(3.5), extent,
                        width=math.radians(7), bottom=8, color='#E47833', linewidth=0.1, alpha=0.17)
                if label_blocks:
                    ax1.annotate(blockname[3], (ephem.hours(block["RA"]) - math.radians(1.5), extent + 0.15), size=15,
                                 color='r')

    # what if try adding a ring around the whole thing with alpha gradient to get the wedge-edge fadeout?
    inner = plt.Circle(58)
    outer = plt.Circle(65)
    # plt.PatchCollection?

    # plot_ossos_discoveries(ax1, discoveries)

    plot_planets_plus_Pluto(ax1)

    if plot_Ijiraq:
        # special detection in 13AE: Ijiraq at 2013-04-09 shows inner limit of sensitivity.
        # Position from Horizons as it's excluded from the list of detections
        ax1.scatter(ephem.hours('14 29 46.57'), 9.805,
                    marker='o', s=4, facecolor='b', edgecolor=plot_fanciness.ALMOST_BLACK, linewidth=0.15, alpha=0.8)
        # ax1.annotate('Ijiraq', (ephem.hours('14 29 46.57') + math.radians(7), 9.8 + 2), size=5)

    ra, dist, hlat, Hmag = parsers.synthetic_model_kbos(kbotype='resonant', arrays=True, maglimit=24.7)
    # can't plot Hmag as marker size in current setup.
    ax1.scatter(ra, dist, marker='o', s=2, facecolor=plot_fanciness.ALMOST_BLACK,
                edgecolor=plot_fanciness.ALMOST_BLACK, linewidth=0.1, alpha=0.12, zorder=1)

    plt.draw()
    outfile = 'topdown_RA_d_OSSOS_v{}.pdf'.format(parameters.RELEASE_VERSION)
    plt.savefig(outfile, transparent=True, bbox_inches='tight')

    return


def plot_planets_plus_Pluto(ax, date=parameters.NEWMOONS[parameters.DISCOVERY_NEW_MOON]):
    for planet in [ephem.Saturn(), ephem.Uranus(), ephem.Neptune(), ephem.Pluto()]:
        planet.compute(ephem.date(date))
        fc = plot_fanciness.ALMOST_BLACK
        if planet.name == 'Pluto':
            alpha = 0.35
            size = 10
            fs = 5
        else:
            alpha = 0.7
            size = 20
            fs = 10
        ax.scatter(planet.ra, planet.sun_distance,
                   marker='o',
                   s=size,
                   facecolor=fc,
                   edgecolor=fc,
                   alpha=alpha)
        if planet.name != 'Saturn':
            ax.annotate(planet.name, (planet.ra - (math.radians(0.5)), planet.sun_distance + 2), size=fs)
        else:
            ax.annotate(planet.name, (planet.ra + (math.radians(10)), planet.sun_distance - 2), size=fs)
        # plot Neptune's orbit: e is 0.01 so can get away with a circle
        if planet.name == 'Neptune':
            orb = np.arange(0, 2 * np.pi, (2 * np.pi) / 360)
            ax.plot(orb, np.repeat(planet.sun_distance, len(orb)), color='b', linestyle=':', linewidth=0.4, alpha=0.7)

    return


def plot_ossos_discoveries(ax, discoveries, lpmag=False):
    """
    these are all being plotted at their discovery locations now,
    which are provided by the Version Releases in decimal hours.
    """
    # need to make this row-wise
    fc = ['b', '#E47833']
    alph = 0.85
    marker = ['o', 'd']
    size = [7, 11]
    pl_index = np.where((discoveries['cl'] == 'res') & (discoveries['j'] == 3) & (discoveries['k'] == 2))
    l = []
    for k, n in enumerate(discoveries):
        if k not in pl_index[0]:
            l.append(k)
    not_plutinos = discoveries[l]
    plutinos = discoveries[pl_index]
    for j, d in enumerate([not_plutinos, plutinos]):
        ra = [ephem.hours(str(n)) for n in d['ra_dis']]
        ax.scatter(ra, d['dist'],
                   marker=marker[j], s=size[j], facecolor=fc[j], edgecolor='w', linewidth=0.25,
                   alpha=alph, zorder=2)  # original size=4.5

    # for obj in discoveries:
    # if lpmag:
    #         if obj.mag <= lpmag:  # show up the bright ones for Gemini LP
    #             fc = 'r'
    #     ra = ephem.hours(str(obj.ra_discov))
    #     if (obj.classification == 'res' and obj.n == 3 and obj.m == 2):
    #         print obj.name
    #         ax.scatter(ra, obj.dist,
    #                    marker='o', s=15 - obj.H, facecolor='#E47833', edgecolor='w', linewidth=0.4,
    #                    alpha=alph)  # original size=4.5
    #     else:
    #         # scaling the size of marker by the absolute magnitude of each object (so size++ with --H, as needed).
    #         ax.scatter(ra, obj.dist,
    #                    marker='o', s=15 - obj.H, facecolor=fc, edgecolor='w', linewidth=0.4,
    #                    alpha=alph)  # original size=4.5

    return


def side_elevation_SolarSystem(date="2013/04/09 08:50:00"):
    fig = plt.figure(figsize=(6, 2))
    ax = fig.add_subplot(111)  # , aspect="equal")

    # viewpoint is from RA 18 hrs (have to ensure that)
    # want to plot the L7 with their inclinations
    # and the OSSOS blocks with the ecliptic latitudes = inclinations, again as wedges

    # need to think about converting RA properly to ecliptic lat/lon

    ax.set_xlim(-60, 60)
    ax.set_ylim(-40, 40)
    plt.ylabel('inclination (degrees)')  # proxy for Ecliptic latitude (degrees)
    plt.xlabel('AU')

    plt.draw()
    outfile = 'side_elevation_20130409'
    plt.savefig(outfile + '.pdf', transparent=True)


def sky_map_global():
    subplot(111, projection='aitoff')
    grid(True)
    # coords in radians
    box = Rectangle((radians(30), radians(30)), radians(15), radians(30))
    gca().add_patch(box)
    draw()

    return


def orbit_fit_residuals(discoveries, blockID='O13AE'):
    """Brett: What are relevant and stunning are the actual orbit fit residuals
    for orbits with d(a)/a<1%. I would be happy with a simple histogram of all those numbers,
    although the 'tail' (probably blended and not yet completely flagged) should
    perhaps be excluded.  There will be a pretty clear gaussian I would think, peaking near 0.07".
    """
    ra_residuals = []
    dec_residuals = []
    ressum = []
    for i, orbit in enumerate(discoveries):
        if (orbit.da / orbit.a) < 0.01:
            print i, len(discoveries), orbit.observations[0].provisional_name, orbit.da / orbit.a
            res = orbit.residuals.split('\n')
            for r in res:
                rr = r.split(' ')
                if (len(rr) > 1) and not (rr[0].startswith('!')):
                    ra_residuals.append(float(r.split(' ')[4]))
                    dec_residuals.append(float(r.split(' ')[5]))
                    ressum.append(1000 * (float(r.split(' ')[4]) ** 2 + float(r.split(' ')[5]) ** 2))

    plt.figure(figsize=(6, 6))
    bins = [r for r in range(-25, 150, 25)]
    n, bins, patches = plt.hist(ressum, histtype='step', color='b', bins=bins, label='$dra^{2}+ddec^{2}$')
    # midpoints = [r/1000. for r in range(-350,350,50)]
    # popt, pcov = curve_fit(gaussian, midpoints, n)
    # print 'a', popt[0], 'mu', popt[1], 'sigma', popt[2]
    # xm = np.linspace(-.375, .375, 100)  # 100 evenly spaced points
    # plt.plot(xm, gaussian(xm, popt[0], popt[1], popt[2]), ls='--', color='#E47833', linewidth=2)
    # sigma = r'$\sigma = %.2f \pm %.2f$' % (popt[2], np.sqrt(pcov[2, 2]))
    # plt.annotate(sigma, (0.12, 50), color='#E47833')
    plt.xlim((-25, 150))  # sufficient for 13AE data
    plt.ylabel('observations of orbits with d(a)/a < 1% (i.e 3-5 month arcs)')
    plt.xlabel('orbit fit residuals (milliarcsec)')
    plt.legend()
    # clean_legend()
    plt.draw()
    outfile = 'orbit_fit_residuals_13AE_corrected'
    plt.savefig(outfile + '.pdf', transparent=True)

    return


def gaussian(x, a, b, c):
    val = a * np.exp(-(x - b) ** 2 / c ** 2)
    return val


def delta_a_over_a(discoveries):
    arclen = []
    da_over_a = []
    for orbit in discoveries:
        if (orbit.da / orbit.a) < 0.01:
            da_over_a.append((orbit.da / orbit.a) * 100)
            arclen.append(orbit.arc_length)

    plt.figure(figsize=(6, 6))
    plt.scatter(arclen, da_over_a, marker='o', facecolor='b', edgecolor=plot_fanciness.ALMOST_BLACK,
                linewidth=0.15, alpha=0.5)
    plt.xlabel('arc length of orbit (days)')
    plt.ylabel('d(a)/a (percent)')
    plot_fanciness.remove_border()
    plt.draw()
    outfile = 'delta_a_over_a_13AE_corrected'
    plt.savefig(outfile + '.pdf', transparent=True)
    print 'objects plotted:', len(arclen)


def main():
    # parsers.output_discoveries_for_animation()
    discoveries = parsers.ossos_release_parser()
    top_down_SolarSystem(discoveries, plot_blocks=True, future_blocks=False, plot_Ijiraq=True,
                         label_blocks=True)

# orbit_fit_residuals(discoveries)
# delta_a_over_a(discoveries)


if __name__ == "__main__":
    main()





__author__ = 'Michele Bannister   git:@mtbannister'

import sys

import numpy
import matplotlib.pyplot as plt
from astropy.table import Table

import plot_fanciness
import parameters

path = '/Users/michele/Dropbox/Papers in progress/OSSOS/First_quarter/data/SSim_fit/'

fig, ax = plt.subplots(1, 1)
ls = ['d', 'k--', 'r-.']

for i, fn in enumerate(['EO-a-i.detections',
                        'L7-EO-a-i.sim',
                        'NoKernel-EO-a-i.sim']):
    with open(path + fn, 'r') as infile:
        raw = Table.read(infile, format='ascii', delimiter=' ', data_start=1, guess=False,
                         header_start=0)['a']
        # create a cumulative fraction
        raw.sort()
        bins = range(400, 470, 1)
        bins = [n / 10. for n in bins]
        hist, bin_edges = numpy.histogram(raw, bins=bins)
        cumulative = []
        # plotting differs for the actual detections as they are far less numerous: sample sparsely
        if i == 0:
            for k, r in enumerate(raw):
                cumulative.append(float(k + 1) / float(len(raw)))
            ax.scatter(raw, cumulative,
                       marker=ls[i], s=15, alpha=0.7, label='OSSOS, this work')
        else:
            for j, n in enumerate(hist):
                cumulative.append(float(sum(hist[:j])) / float(len(raw)))
            if fn == 'L7-EO-a-i.sim':
                label = 'L7 model'
            if fn == 'NoKernel-EO-a-i.sim':
                label = 'L7 model, no kernel'
            ax.plot(bins[:len(bins) - 1], cumulative, ls[i], alpha=0.8, label=label)

plt.ylabel('fraction with semimajor axis $<$ a')
plt.xlabel('semimajor axis (AU)')
plt.xlim([42.4, 47])
plt.ylim([0.0, 1.05])
plt.grid(True, which='both', alpha=0.4)
plt.legend(loc='lower right', scatterpoints=1)
plot_fanciness.remove_border(ax)
plt.draw()
# plt.show()
outfile = 'ssim_v{}.pdf'.format(parameters.RELEASE_VERSION)
plt.savefig(outfile, transparent=True, bbox_inches='tight')
sys.stdout.write('{}\n'.format(outfile))



from unittest import TestCase
from mpc import Observation

__author__ = 'jjk'
ted_string = """2011 04  28.39916  18 32 37.019  -21 13 50.20  26.0R NI100     304"""
mpc_string = """     NI100     2011 04 28.39916 18 32 37.019-21 13 50.20         26.0 R      304"""


class TestObservation(TestCase):
    def test_from_ted(self):
        mpc = Observation.from_string(mpc_string)
        mpc.comment = None
        ted = Observation.from_ted(ted_string)
        ted.comment = None
        self.assertEquals(ted, mpc)



__author__ = "David Rusk <drusk@uvic.ca>"

from ossos.fitsviewer.baseviewer import WxMPLFitsViewer
from ossos.fitsviewer.displayable import DisplayableImageTriplet


class TripletViewer(WxMPLFitsViewer):
    """
    Displays a single FITS image at a time.
    """

    def mark_sources(self, cutout_grid):
        for frame_index in range(cutout_grid.num_frames):
            self.mark_frame(cutout_grid, frame_index)

    def mark_frame(self, cutout_grid, frame_index):
        focus_cutout = cutout_grid.get_cutout(frame_index, frame_index)

        for time_index in range(cutout_grid.num_times):
            cutout = cutout_grid.get_cutout(frame_index, time_index)

            x, y = focus_cutout.pixel_source_point
            offset_x, offset_y = focus_cutout.reading.get_coordinate_offset(cutout.reading)
            x += offset_x
            y += offset_y

            fwhm = float(cutout.reading.get_observation_header()["FWHM"])
            radius = 2 * round(fwhm)

            self.current_displayable.get_singlet(frame_index, time_index).place_marker(x, y, radius)

    def _create_displayable(self, cutout_grid):
        return DisplayableImageTriplet(cutout_grid, self.ds9)



import math
import sys
import argparse
import os

import numpy as np
import ephem
from astropy.io import votable
from astropy.time import Time, TimeDelta
import Polygon
from matplotlib.patches import Rectangle
from matplotlib.font_manager import FontProperties
import matplotlib.pyplot as plt

from planning import megacam
import parsers
import parameters
import plot_fanciness
import plot_objects
from planning import invariable




# from ossos import cameras  # bit over the top to show all the ccds?

plot_extents = {"13AE": [209.8, 218.2, -15.5, -9.5],
                "13AO": [235.4, 243.8, -15.5, -9.5],
                "13A": [209, 244, -22, -9],  # for both 13A blocks together
                "15AM": [229, 235, -15, -9],
                "15AP": [198, 207, -12, -4],
                "13B": [30, 5, -2, 18],  # for both 13B blocks together
                "13BL": [9, 18, 1, 7],
                "14BH": [18, 27, 10, 16],
                "15BD": [44, 54, 13, 20],
                "15BS": [7, 16, -4, 4]
}

xgrid = {'2013': [-3, -2, -1, 0, 1, 2, 3],
         '2014r': [-3.5, -2.5, -1.5, -0.5, 0.5, 1.5, 2.5, 3.5],
         '2014': [-3, -2, -1, 0, 1, 2, 3],
         'astr': [1, 2, 3, 4]}

ygrid = {'2013': [-1, 0, 1],
         '2014': [-1, 0, 1],
         'astr': [-2.5, -1.5, -0.5, 0.5, 1.5],
         '2014r': [-1.5, -0.5, 0.5, 1.5]}

block_centre = ephem.Ecliptic(0, 0)
field_centre = ephem.Ecliptic(0, 0)

# the offset (in RA and DEC) between fields
field_offset = math.radians(1.00)
# the size of the field
camera_dimen = 0.98

years = {"2014": {"ra_off": ephem.hours("00:00:00"),
                  "dec_off": ephem.hours("00:00:00"),
                  "fill": False,
                  "facecolor": 'k',
                  "alpha": 0.5,
                  "color": 'b'},
         "2014r": {"ra_off": ephem.hours("00:05:00"),
                   "dec_off": ephem.degrees("00:18:00"),
                   "alpha": 0.5,
                   "fill": False,
                   "facecolor": 'k',
                   "color": 'g'},
         'astr': {"ra_off": ephem.hours("00:00:00"),
                  "dec_off": ephem.degrees("-00:12:00"),
                  "alpha": 0.05,
                  "fill": True,
                  'facecolor': 'k',
                  "color": 'none'}
}

# FIXME: redo this to have brewermpl colours?
def colsym():
    # needs a light grey to get the original populations shown as background.
    mcolours = ['0.75', 'k', 'yellow', 'black', 'blue']  # closer to 1 is lighter. 0.85 is good.
    msymbols = ['o', 'o', 'o', 'x', 'd']

    return mcolours, msymbols


def basic_skysurvey_plot_setup(from_plane=ephem.degrees('0')):
    # # EmulateApJ columnwidth=245.26 pts
    # fig_width_pt = 246.0
    # inches_per_pt = 1.0 / 72.27
    # golden_mean = (sqrt(5.) - 1.0) / 2.0
    # fig_width = fig_width_pt * inches_per_pt
    # fig_height = fig_width * golden_mean * 1.5
    # fig_size = [fig_width, fig_height]

    fig = plt.figure()
    ax = fig.add_subplot(111)  # , aspect="equal")

    handles = []  # this handles the creation of the legend properly at the end of plotting
    labels = []
    handles, labels = plot_galactic_plane(handles, labels)
    # handles, labels = plot_ecliptic_plane(handles, labels)
    handles, labels = plot_invariable_plane(handles, labels, from_plane)

    fontP = FontProperties()
    fontP.set_size('small')  # make the fonts smaller
    plt.xlabel('RA (deg)')
    plt.ylabel('Declination (deg)')
    plt.grid(True, which='both')
    # plot_fanciness.remove_border(ax)

    return handles, labels, ax, fontP


def plot_galactic_plane(handles, labels):
    gp = [ephem.Galactic(str(lon), str(0)) for lon in range(0, 360)]
    galPlane = [ephem.Equatorial(coord) for coord in gp]
    # rewrap the ra to start at zero: that otherwise causes a problem in the plotting!
    temp = [(math.degrees(coord.ra), math.degrees(coord.dec)) for coord in galPlane]
    temp.sort(key=lambda x: x[0])
    ra_galPlane = [t[0] for t in temp]
    dec_galPlane = [tt[1] for tt in temp]
    handles.append(plt.plot(ra_galPlane, dec_galPlane, 'b'))
    labels.append('galactic plane')
    # plt.plot([r+360 for r in ra_galPlane], dec_galPlane, 'b')  # echo

    return handles, labels


def plot_ecliptic_plane(handles, labels):
    ep = [ephem.Ecliptic(str(lon), str(0)) for lon in
          range(0, 360)]  # this line is fine: need Ecliptic(long, lat) format in creation
    ecPlane = [ephem.Equatorial(coord) for coord in ep]  # so this must be where the issue is.
    ra_ecPlane = [math.degrees(coord.ra) for coord in ecPlane]
    dec_ecPlane = [math.degrees(coord.dec) for coord in ecPlane]
    handles.append(plt.plot(ra_ecPlane, dec_ecPlane, '#E47833'))
    labels.append('ecliptic')
    # plt.plot([rr+360 for rr in ra_ecPlane], dec_ecPlane, '#E47833')  # echo

    return handles, labels


def plot_invariable_plane(handles, labels, from_plane=ephem.degrees('0')):
    # Plot the invariable plane: values from DE405, table 5, Souami and Souchay 2012
    # http://www.aanda.org/articles/aa/pdf/2012/07/aa19011-12.pdf
    # Ecliptic J2000
    lon = np.arange(-2 * math.pi, 2 * math.pi, 0.5 / 57)
    lat = 0 * lon
    (lat, lon) = invariable.trevonc(lat, lon)

    ec = [ephem.Ecliptic(x, y) for (x, y) in np.array((lon, lat)).transpose()]
    eq = [ephem.Equatorial(coord) for coord in ec]
    handles.append(plt.plot([math.degrees(coord.ra) for coord in eq],
                            [math.degrees(coord.dec) for coord in eq],
                            ls='--',
                            color=plot_fanciness.ALMOST_BLACK,
                            lw=1,
                            alpha=0.7))
    labels.append('invariable')

    return handles, labels


def build_ossos_footprint(ax, block_name, block, field_offset, plot=True, plot_col='b'):
    # build the pointings that will be used for the discovery fields
    x = []
    y = []
    names = []
    coverage = []
    year = '2014'
    if year == 'astr':
        field_offset = field_offset * 0.75

    sign = -1
    # if 'f' in block:  # unsure what this was ever meant to do
    # sign = 1
    rac = ephem.hours(block["RA"]) + years[year]["ra_off"]
    decc = ephem.degrees(block["DEC"]) + sign * years[year]["dec_off"]
    width = field_offset / math.cos(decc)
    block_centre.from_radec(rac, decc)
    block_centre.set(block_centre.lon + xgrid[year][0] * width, block_centre.lat)
    field_centre.set(block_centre.lon, block_centre.lat)
    for dx in xgrid[year]:
        (rac, decc) = field_centre.to_radec()
        for dy in ygrid[year]:
            ddec = dy * field_offset
            dec = math.degrees(decc + ddec)
            ra = math.degrees(rac)
            names.append("%s%+d%+d" % ( block_name, dx, dy))
            y.append(dec)
            x.append(ra)
            xcen = ra
            ycen = dec

            dimen = camera_dimen
            coverage.append(Polygon.Polygon((
                (xcen - dimen / 2.0, ycen - dimen / 2.0),
                (xcen - dimen / 2.0, ycen + dimen / 2.0),
                (xcen + dimen / 2.0, ycen + dimen / 2.0),
                (xcen + dimen / 2.0, ycen - dimen / 2.0),
                (xcen - dimen / 2.0, ycen - dimen / 2.0))))

            if plot:
                ax.add_artist(Rectangle(xy=(ra - dimen / 2.0, dec - dimen / 2.0),
                                        height=dimen,
                                        width=dimen,
                                        edgecolor='k',
                                        facecolor=plot_col,
                                        lw=0.5, fill=True, alpha=0.15, zorder=0))

        rac += field_offset / math.cos(decc)
        for i in range(3):
            field_centre.from_radec(rac, decc)
            field_centre.set(field_centre.lon, block_centre.lat)
            (ttt, decc) = field_centre.to_radec()

    ras = np.radians(x)
    decs = np.radians(y)

    return ras, decs, coverage, names, ax


def synthetic_model_kbos(coverage, input_date=parameters.DISCOVERY_NEW_MOON):
    # # build a list of KBOs that will be in the discovery fields.

    raise NotImplementedError('Not yet fully completed.')

    # kbos = parsers.synthetic_model_kbos(input_date)
    # ## keep a list of KBOs that are in the discovery pointings
    #
    #     # FIXME
    #     for field in coverage:
    #         for kbo in kbos:
    #             if kbo.isInside(ra[-1], dec[-1]):
    #                 kbos.append(kbo)
    #                 break
    #
    # return ra, dec, kbos


def keplerian_sheared_field_locations(ax, kbos, date, ras, decs, names, elongation=False, plot=False):
    """
    Shift fields from the discovery set to the requested date by the average motion of L7 kbos in the discovery field.
    :param ras:
    :param decs:
    :param plot:
    :param ax:
    :param kbos: precomputed at the discovery date for that block. e.g. Oct new moon for 13B
    :param date:
    :param names:
    :param elongation:
    """
    seps = {'dra': 0., 'ddec': 0.}
    for kbo in kbos:
        ra = kbo.ra
        dec = kbo.dec
        kbo.compute(date)
        seps['dra'] += kbo.ra - ra
        seps['ddec'] += kbo.dec - dec

    seps['dra'] /= float(len(kbos))
    seps['ddec'] /= float(len(kbos))

    print date, seps, len(kbos)

    for idx in range(len(ras)):
        name = names[idx]
        ra = ras[idx] + seps['dra']
        dec = decs[idx] + seps['ddec']
        if plot:
            ax.add_artist(Rectangle(xy=(math.degrees(ra) - camera_dimen / 2.0, math.degrees(dec) - camera_dimen / 2.0),
                                    height=camera_dimen,
                                    width=camera_dimen,
                                    edgecolor='b',
                                    facecolor='b',
                                    lw=0.5, fill=True, alpha=0.2))
        if elongation:
            # For each field centre, plot the elongation onto the field at that date.
            elong = field_elongation(ephem.degrees(ra), ephem.degrees(dec), date)
            ax.annotate(name, (math.degrees(ra) + camera_dimen / 2., math.degrees(dec)), size=3)
            ax.annotate("%0.1f" % elong, (math.degrees(ra) + camera_dimen / 4., math.degrees(dec) - camera_dimen / 4.),
                        size=5)

    return ax


def field_elongation(ra, dec, date):
    """
    For a given field, calculate the solar elongation at the given date.
    :param ra: field's right ascension. unit="h"   format="RAh:RAm:RAs"
    :param dec: field's declination. degrees
    :param date: date at which to calculate elongation
    :return: elongation from the Sun in degrees
    """
    sun = ephem.Sun()
    sun.compute(date)
    sep = ephem.separation((ra, dec), sun)
    retval = 180. - math.degrees(sep)

    return retval


def plot_USNO_B1_stars(ax):
    t = votable.parse(usnoB1.TAPQuery(ra_cen, dec_cen, width, height)).get_first_table()
    Rmag = t.array['Bmag'][t.array['Bmag'] < 15]
    min = max(Rmag.min(), 11)
    max = Rmag.max()
    scale = 0.5 * 10 ** ((min - Rmag) / 2.5)
    print scale.min(), scale.max()
    ax.scatter(t.array['RAJ2000'], t.array['DEJ2000'], s=scale, marker='o', facecolor='y', alpha=0.8, edgecolor='',
               zorder=-10)

    return ax


def plot_existing_CFHT_Megacam_observations_in_area(ax, qra, qdec, discovery=None, obs_ids=None):
    filename = '/Users/michele/' + "megacam{:+6.2f}{:+6.2f}.xml".format(qra, qdec)
    if not os.access(filename, os.R_OK):
        data = megacam.TAPQuery(qra, qdec, 60.0, 30.0).read()
        fobj = open(filename, 'w')
        fobj.write(data)
        fobj.close()
    fobj = open(filename, 'r')
    t = votable.parse(fobj).get_first_table()

    ra = t.array['RAJ2000']
    dec = t.array['DEJ2000']
    mjdate = t.array['MJDATE']

    camera_dimen = 0.98

    # More sophisticated date ID on plot. Let's identify first-year, discovery, second-year followup
    for idx in xrange(ra.size):
        alpha = 0.1
        # print abs(Time(mjdate[idx], format='mjd') - discovery) < TimeDelta(1, format='jd')
        if Time(mjdate[idx], format='mjd') < Time('2014-01-01'):
            ec = 'b'
        elif (discovery is not None and abs(Time(mjdate[idx], format='mjd') - discovery) < TimeDelta(1, format='jd')):
            print Time(mjdate[idx], format='mjd').iso
            ec = 'r'
            alpha = 1
        else:
            ec = '#E47833'
        r = Rectangle(xy=(ra[idx] - camera_dimen / 2.0, dec[idx] - camera_dimen / 2.0),
                      height=camera_dimen,
                      width=camera_dimen,
                      edgecolor=ec,
                      alpha=alpha,
                      lw=0.1,
                      zorder=-100,
                      fill=False)
        ax.add_artist(r)

    return ax


def plot_discovery_uncertainty_ellipses(ax, date):
    # plot objects at time of triplet with an X, location each 3 days with a single grey pixel, actual observations
    # with a solid dot. Will give suitable corkscrew effect.
    # plot wireframe of footprint at discovery triplet
    # plot hollow circle as location at start of dark run, ellipse (no fill I think) for orbit uncertainty.
    raise NotImplementedError
    return ax


def saving_closeout(blockname, date, extents, file_id):
    # plt.title(date)
    plt.axis(extents)  # depends on which survey field is being plotted
    plt.draw()
    outfile = blockname + file_id + date.split(' ')[0].replace('/', '-') + '.pdf'
    # Always use axes.set_rasterized(True) if you are saving as an EPS file.
    print 'Saving file.', outfile
    plt.savefig(outfile, transparent=True)
    plt.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-opposition",
                        help="Plots blocks at their opposition dates (the date .")
    parser.add_argument("-discovery",
                        action="store_false",
                        help="Plots blocks at their discovery dates.")
    parser.add_argument("-date",
                        help="Plot blocks at a specific user-provided date, format yyyy/mm/dd HH:MM:SS.")
    parser.add_argument("-elongation",
                        help="Display text of block IDs and arc distance of blocks from opposition on the given date")
    parser.add_argument("-synthetic",
                        action="store_true",
                        help="Use L7 model to slew blocks, or plot L7 model objects that fall in a given block.")
    parser.add_argument("-existing",
                        action="store_true",
                        help="Plot all OSSOS imaging of the block.")
    parser.add_argument("blocks",
                        nargs='*',
                        help="specify blocks to be plotted, e.g. 13AE. Without specifying, will do all eight blocks,"
                             "making eight separate plots.")
    args = parser.parse_args()
    print(args)

    file_id = ""
    if args.blocks:  # Coordinates at discovery-opposition of currently-determined OSSOS blocks (2013: 4; 2015: 8
    # total).
        blocks = {}
        for b in args.blocks:
            assert b in parameters.BLOCKS
            blocks[b] = parameters.BLOCKS[b]
    else:  # do them all!
        blocks = parameters.BLOCKS

    discoveries = parsers.ossos_release_parser()  # ossos_release_parser()

    # extent = plot_extents['13A'] # hardwired for doing both at once

    for blockname, block in blocks.items():
        extent = plot_extents[blockname]

        if args.opposition:
            # 13AE is a month different in opposition from 13AO, but 13B are both in October
            date = parameters.OPPOSITION_DATES[blockname]
            file_id = 'opposition-'

        if args.discovery:
            # E block discovery triplets are April 4,9,few 19; O block are May 7,8.
            # BL is 09-29 AND 10-31. Discoveries in 10-31 are placed at predicted locations, to accommodate split.
            date = parameters.DISCOVERY_DATES[blockname]
            file_id = 'discovery-'

        if args.date:
            date = args.date
            args.synthetic = True
            file_id = '-at-date-'

        if args.elongation:
            file_id = 'elongation-'

        if args.existing:
            file_id += '-bg-imaging-'

        assert date
        print blockname, date

        if blockname == '13AO':
            handles, labels, ax, fontP = basic_skysurvey_plot_setup(from_plane=ephem.degrees('6.0'))
            ras, decs, coverage, names, ax = build_ossos_footprint(ax, blockname, block, field_offset, plot=True,
                                                                   plot_col='#E47833')
        else:
            handles, labels, ax, fontP = basic_skysurvey_plot_setup()
            ras, decs, coverage, names, ax = build_ossos_footprint(ax, blockname, block, field_offset, plot=True)
        ax, kbos = plot_objects.plot_known_tnos_singly(ax, extent, date)
        if args.synthetic:  # defaults to discovery new moon: FIXME: set date appropriately
            ra, dec, kbos = synthetic_model_kbos(coverage)
            ax = keplerian_sheared_field_locations(ax, kbos, date, ras, decs, names,
                                                   elongation=args.elongation, plot=True)
            ax = plot_objects.plot_synthetic_kbos(ax, coverage)

        block_discoveries = discoveries[
            np.array([name.startswith('o3' + blockname[-1].lower()) for name in discoveries['object']])]
        ax = plot_objects.plot_ossos_discoveries(ax, block_discoveries, blockname,
                                                 prediction_date=date)  # will predict for dates other than discovery

        # ax = plot_objects.plot_planets(ax, extent, date)

        # ax = plot_discovery_uncertainty_ellipses(ax, date)
        # if blockname == '13AE':  # special case: Saturn was adjacent at discovery
        #     ax = plot_objects.plot_saturn_moons(ax)

        if args.existing:
            plot_existing_CFHT_Megacam_observations_in_area(ax)

        saving_closeout(blockname, date, plot_extents[blockname], file_id)
    # saving_closeout('13A', date, extent, file_id)

    sys.stderr.write("Finished.\n")





import logging

from astropy import units
from astropy.coordinates import SkyCoord

from ..astrom import SourceReading
from ..downloads.cutouts.focus import SingletFocusCalculator
from ..downloads.cutouts.source import SourceCutout
from ..gui import logger

__author__ = "David Rusk <drusk@uvic.ca>"


class WxMPLFitsViewer(object):
    """
    Display FITS images using ds9.
    """

    def __init__(self, parent, display):
        self.parent = parent

        self.current_cutout = None
        self.current_displayable = None
        self._ds9 = display
        self._displayables_by_cutout = {}
        self.mark_source = True
        self.mark_prediction = False

    @property
    def ds9(self):
        return self._ds9

    def display(self, cutout, use_pixel_coords=False):
        """
        :param cutout: source cutout object to be display
        :type cutout: source.SourceCutout
        """
        logging.debug("Current display list contains: {}".format(self._displayables_by_cutout.keys()))
        logging.debug("Looking for {}".format(cutout))
        assert isinstance(cutout, SourceCutout)
        if cutout in self._displayables_by_cutout:
            displayable = self._displayables_by_cutout[cutout]
        else:
            displayable = self._create_displayable(cutout)
            self._displayables_by_cutout[cutout] = displayable

        self._detach_handlers(self.current_displayable)

        self.current_cutout = cutout
        self.current_displayable = displayable

        self._attach_handlers(self.current_displayable)

        self._do_render(self.current_displayable)

        self.mark_apertures(cutout, pixel=use_pixel_coords)

        self.draw_uncertainty_ellipse(cutout)

    def clear(self):
        self.ds9.set("frame delete all")

    def draw_uncertainty_ellipse(self, cutout):
        """
        Draws an ErrEllipse with the spcified dimensions.  Only one ErrEllipse can be drawn and
        only once (not movable).
        """
        if not self.mark_prediction:
            return
        try:
            colour = cutout.reading.from_input_file and 'b' or 'g'
            colour = cutout.reading.null_observation and 'c' or colour
            dash = cutout.reading.null_observation and 1 or 0
            sky_coord = cutout.reading.sky_coord

            # RA/DEC value of observation / prediction goes to X/Y  using PV keywords
            x, y, extno = cutout.world2pix(sky_coord.ra.to(units.degree).value,
                                           sky_coord.dec.to(units.degree).value, usepv=True)


            # assert cutout.extno == extno
            # The X/Y pixel values are then converted to RA/DEC without PV for use with DS9.
            ra, dec = cutout.pix2world(x, y, usepv=False)
            uncertainty_ellipse = cutout.reading.uncertainty_ellipse

            self.current_displayable.place_ellipse((ra, dec), uncertainty_ellipse, colour=colour, dash=dash)

            # Also mark on the image where source is localted, not just the ellipse.
            ra, dec = cutout.pix2world(cutout.pixel_x, cutout.pixel_y, usepv=False)

            self.current_displayable.place_marker(ra, dec, radius=8, colour=colour)
        except Exception as ex:
            print "EXCEPTION: {}".format(ex)
            logger.debug(type(ex))
            logger.debug(str(ex))

    def refresh_markers(self):
        self.mark_apertures(self.current_cutout)

    def mark_apertures(self, cutout, pixel=False):
        pass

    def release_focus(self):
        self.parent.SetFocus()

    def reset_colormap(self):
        if self.current_displayable is not None:
            self.current_displayable.reset_colormap()

    def toggle_reticule(self):
        if self.current_displayable.toggle_reticule():
            self.mark_apertures(self.current_cutout, pixel=False)
            self.draw_uncertainty_ellipse(self.current_cutout)

    def _attach_handlers(self, displayable):
        pass

    def _detach_handlers(self, displayable):
        pass

    def _create_displayable(self, cutout):
        raise NotImplementedError()

    def align(self, cutout, reading, source):
        """
        Set the display center to the reference point.

        @param cutout:  The cutout to align on
        @type cutout: SourceCutout
        @param reading:  The reading this cutout is from
        @type reading: SourceReading
        @param source: The source the reading is from
        @type source: Source
        @return:
        """
        if not self.current_displayable:
            return
        if not self.current_displayable.aligned:
            focus_sky_coord = reading.reference_sky_coord
            #focus_calculator = SingletFocusCalculator(source)
            #logger.debug("Got focus calculator {} for source {}".format(focus_calculator, source))
            #focus = cutout.flip_flip(focus_calculator.calculate_focus(reading))
            #focus = cutout.get_pixel_coordinates(focus)
            #focus = cutout.pix2world(focus[0], focus[1], usepv=False)
            #focus_sky_coord = SkyCoord(focus[0], focus[1])
            self.current_displayable.pan_to(focus_sky_coord)

    @staticmethod
    def _do_render(displayable):
        displayable.render()



import re
import sys

import ephem


# Number_Mil={'B': 110000, 'C': 120000, 'D': 130000, 'E': 140000, 'F': 150000}
#Number_Cent={'J': 1900, 'K': 2000}
def date_unpack(pdate):
    (yyyy, mm, dd) = (2000, 01, 01)
    try:
        YY = {'I': 1800, 'J': 1900, 'K': 2000}
        Ncode = '0123456789ABCDEFGHIJKLMNOPQRSTUV'
        yyyy = YY[pdate[0]] + int(pdate[1:3])
        mm = Ncode.rindex(pdate[3])
        dd = float(Ncode.rindex(pdate[4]))
    except:
        sys.stderr.write("ERROR converting date part {}:".format(pdate))
    return (yyyy, mm, dd)


def desig_unpack(desig):
    if re.match('\d+', desig):
        return str(int(desig))
    Kilo = 'ABCDEFGHIJKLMNOPQRSTUV'
    j = Kilo.rfind(desig[0])
    if j != -1:
        if re.match('^\d+$', desig[1:7]):
            return str(100000 + j * 10000 + int(desig[1:7]))
    YY = {'I': 1800, 'J': 1900, 'K': 2000}
    try:
        yyyy = str(YY[desig[0]] + int(desig[1:3]))
    except KeyError as e:
        return desig
    Ncode = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
    Mcode = desig[3] + desig[6]
    cycle = Ncode.rindex(desig[4]) * 10 + int(desig[5])
    if (cycle > 0):
        cycle = str(cycle)
    else:
        cycle = ''
    return yyyy + ' ' + Mcode + cycle


def getKBOs(mpc_file, cond='a > 30'):
    f = open(mpc_file)
    lines = f.readlines()
    f.close()

    # while "------" not in lines[0]:
    #     lines.pop(0)
    lines.pop(0)
    nobj = 0
    lineCount = 0
    kbos = []
    for line in lines:
        lineCount = lineCount + 1
        if line[0] == '#' or len(line) < 103 or line[0:3] == '---':
            continue
        kbo = ephem.EllipticalBody()

        if len(line[8:13].strip()):
            kbo._H = float(line[8:13])
            kbo._G = float(line[14:19])
        else:
            kbo._H = 20
            kbo._G = 0
        arc = line[127:136]
        try:
            if 'days' in arc:
                arc = int(arc.split()[0]) / 365.25
            else:
                arc = -eval(arc)
        except:
            arc = 0
        kbo._epoch_M = date_unpack(line[20:25].strip())
        kbo._M = float(line[26:35].strip())
        kbo._om = float(line[37:46].strip())
        kbo._Om = float(line[48:57].strip())
        kbo._inc = float(line[59:68].strip())
        kbo._e = float(line[70:79].strip())
        kbo._epoch = '2011/08/01'
        kbo._a = float(line[92:103].strip())
        kbo.compute()

        a = kbo._a
        q = kbo._a * (1 - kbo._e)
        H = kbo._H

        if eval(cond):
            kbo.name = desig_unpack(line[0:7].strip())
            kbos.append(kbo)

    return kbos



__author__ = 'jjk'

import ctypes
import datetime
import logging
import math
from StringIO import StringIO
import tempfile

import numpy
from astropy.coordinates import SkyCoord
from astropy import units
from astropy.units.quantity import Quantity

import mpc
from mpc import Time

LIBORBFIT = "/usr/local/lib/liborbfit.so"


class OrbfitError(Exception):
    def __init__(self):
        super(OrbfitError, self).__init__(
            "Insufficient observations for an orbit.")


class Orbfit(object):
    """
    This class provides orbital information derived by calling 'fit_radec'.
    """

    def __init__(self, observations):
        """
        Given a list of mpc.Observations, compute the orbit using fit_radec and provide methods for
        accessing that orbit.

        Requires at least 3 mpc.Observations in the list.
        :rtype : Orbfit
        """
        assert isinstance(observations, tuple) or isinstance(observations, list) or isinstance(observations,
                                                                                               numpy.ndarray)

        if len(observations) < 3:
            raise OrbfitError()
        self.orbfit = ctypes.CDLL(LIBORBFIT)
        self.observations = observations
        self._a = self._e = self._inc = self._Node = self._om = self._T = None
        self._da = self._de = self._dinc = self._dNode = self._dom = self._dT = None
        self._coordinate = self._pa = self._dra = self._ddec = self._date = self._time = None
        self._fit_radec()

    def _fit_radec(self):

        _abg_file = tempfile.NamedTemporaryFile(suffix='.abg')
        _mpc_file = tempfile.NamedTemporaryFile(suffix='.mpc')

        # call fit_radec with mpcfile and abgfile
        self.orbfit.fitradec.restype = ctypes.POINTER(ctypes.c_double * 2)
        self.orbfit.fitradec.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
        for observation in self.observations:
            assert isinstance(observation, mpc.Observation)
            if observation.null_observation:
                continue
            obs = observation
            self.name = observation.provisional_name
            ra = obs.ra.replace(" ", ":")
            dec = obs.dec.replace(" ", ":")
            res = getattr(obs.comment, 'plate_uncertainty', 0.2)
            _mpc_file.write("{} {} {} {} {}\n".format(obs.date.jd, ra, dec, res, 568, ))
        _mpc_file.seek(0)
        result = self.orbfit.fitradec(ctypes.c_char_p(_mpc_file.name),
                                      ctypes.c_char_p(_abg_file.name))

        self.distance = result.contents[0] * units.AU
        self.distance_uncertainty = result.contents[1] * units.AU

        # call abg_to_aei to get elliptical elements and their chi^2 uncertainty.
        self.orbfit.abg_to_aei.restype = ctypes.POINTER(ctypes.c_double * 13)
        self.orbfit.abg_to_aei.argtypes = [ctypes.c_char_p]
        result = self.orbfit.abg_to_aei(ctypes.c_char_p(_abg_file.name))

        self._a = result.contents[0] * units.AU
        self._da = result.contents[6] * units.AU
        self._e = result.contents[1] * units.dimensionless_unscaled
        self._de = result.contents[7] * units.dimensionless_unscaled
        self._inc = result.contents[2] * units.degree
        self._dinc = result.contents[8] * units.degree
        self._Node = result.contents[3] * units.degree
        self._dNode = result.contents[9] * units.degree
        self._om = result.contents[4] * units.degree
        self._dom = result.contents[10] * units.degree
        self._T = result.contents[5] * units.day
        self._dT = result.contents[11] * units.day
        self._epoch = result.contents[12] * units.day
        _abg_file.seek(0)
        self.abg = _abg_file.read()

    @property
    def a(self):
        """
        :return: semi-major axis.
        :rtype: Quantity
        """
        return self._a

    @property
    def da(self):
        """
        :return:Uncertainty of orbital semi-major axis.
        :rtype: Quantity
        """
        return self._da

    @property
    def e(self):
        """
        :return: eccentricity.
        :rtype: Quantity
        """
        return self._e

    @property
    def de(self):
        """
        :return: of orbital eccentricity
        :rtype: Quantity
        """
        return self._de

    @property
    def inc(self):
        """
        :return: Inclination
        :rtype: Quantity
        """
        return self._inc

    @property
    def dinc(self):
        """

        :return: Uncertainty of orbital Inclination
        :rtype: Quantity
        """
        return self._dinc

    @property
    def om(self):
        """

        :return: Argument of peri-centre
        :rtype: Quantity
        """
        return self._om

    @property
    def dom(self):
        """
        :rtype : Quantity
        :return: Uncertainty in argument of peri-centre
        """
        return self._dom

    @property
    def Node(self):
        """
        :rtype : Quantity
        :return: Ascending Node
        """
        return self._Node

    @property
    def dNode(self):
        """
        :rtype : Quantity
        :return: The uncertainty in the chi2 fit of ascending Node value.
        """
        return self._dNode

    @property
    def T(self):
        """
        :return: Time of pericentre passage
        :rtype: Quantity
        """
        return self._T

    @property
    def dT(self):
        """
        :return: Uncertainty in time of pericentre passage
        :rtype: Quantity
        """
        return self._dT

    @property
    def epoch(self):
        """
        :return: Uncertainty in time of pericentre passage
        :rtype: Quantity
        """
        return self._epoch


    def residuals(self, overall=False):
        """
        Builds a summary of the residuals of a fit.  This is useful for visually examining the
        goodness of fit for a small number of observations and the impact of adding a few tentative observations.
        :return: A string representation of the residuals between the best fit orbit and the astrometric measurements used.
        :rtype: str
        """
        # compute the residuals (from the given observations)
        _residuals = ""
        overall_resids = []
        for observation in self.observations:
            self.predict(observation.date)
            coord1 = SkyCoord(self.coordinate.ra, self.coordinate.dec)
            coord2 = SkyCoord(observation.coordinate.ra, self.coordinate.dec)
            observation.ra_residual = float(coord1.separation(coord2).arcsec)
            observation.ra_residual = (coord1.ra.degree - coord2.ra.degree) * 3600.0
            coord2 = SkyCoord(self.coordinate.ra, observation.coordinate.dec)
            observation.dec_residual = float(coord1.separation(coord2).arcsec)
            observation.dec_residual = (coord1.dec.degree - coord2.dec.degree) * 3600.0
            overall_resids.append(math.sqrt(observation.ra_residual ** 2 + observation.dec_residual ** 2))
            _residuals += "{:1s}{:12s} {:+05.2f} {:+05.2f} # {}\n".format(
                observation.null_observation, observation.date, observation.ra_residual, observation.dec_residual,
                observation)
        if overall:
            return overall_resids  # values in arcsec
        else:
            return _residuals

    @property
    def coordinate(self):
        """
        A coordinate object that holds the current predicted location of the object. (see self.time/self.date)
        :return: location of the source determined by predict
        :rtype: ICRS
        """
        return self._coordinate

    @property
    def dra(self):
        """
        :return: Major axis of predicted location uncertainty ellipse
        :rtype: Quantity
        """
        return self._dra

    @property
    def ddec(self):
        """
        :return: Minor axis of predicted location uncertainty ellipse
        :rtype: Quantity
        """
        return self._ddec

    @property
    def date(self):
        """
        :return: String representation of the date prediction is for.
        :rtype: str
        """
        return self._date

    @property
    def time(self):
        """
        :return: Time the predicted location was computed for.
        :rtype: time.Time
        """
        return self._time

    @property
    def pa(self):
        """
        :return: The position angle of the uncertainty of the prediction ellipse.
        :rtype: Quantity
        """
        return self._pa

    @property
    def arc_length(self):
        """
        :return: the arc length of the observations used in the current fit of the orbit.
        :rtype: Quantity
        """
        dates = []
        for observation in self.observations:
            dates.append(observation.date.jd)
        return (max(dates) - min(dates)) * units.day

    def __str__(self):
        """

        """

        res = "{:>10s} {:>8s} {:>8s} {:>8s} {:>8s} {:>8s} {:>8s}\n".format(
            self.observations[0].provisional_name.strip(' '),
            "r (AU)",
            "a (AU)",
            "e",
            "Inc.",
            "Node",
            "peri.")
        res += "{:>10s} {:8.2f} {:8.2f} {:8.2f} {:8.2f} {:8.2f} {:8.2f}\n".format("fit",
                                                                                  self.distance,
                                                                                  self.a,
                                                                                  self.e,
                                                                                  self.inc,
                                                                                  self.Node,
                                                                                  self.om)
        res += "{:>10s} {:8.2f} {:8.2f} {:8.2f} {:8.2f} {:8.2f} {:8.2f}\n".format("uncert",
                                                                                  self.distance_uncertainty,
                                                                                  self.da,
                                                                                  self.de,
                                                                                  self.dinc,
                                                                                  self.dNode,
                                                                                  self.dom)

        res += "{:>10s} {:8.2f} ".format("arc", self.arc_length)
        if self.dra is not None:
            res += "ephemeris uncertainty: {:8.2f} {:8.2f} {:8.2f}".format(self.dra, self.ddec, self.pa)
        res += "\n"

        return res

    def predict(self, date, obs_code=568, abg_file=None):
        """
        use the bk predict method to compute the location of the source on the given date.
        @param date: the julian date of interest or an astropy.core.time.Time object.
        @param obs_code: the Minor Planet Center observatory location code (Mauna Kea: 568 is the default)

        this methods sets the values of coordinate, dra (arcseconds), ddec (arcseconds), pa, (degrees) and date (str)
        """

        if not isinstance(date, Time):
            if isinstance(date, float):
                try:
                    date = Time(date, format='jd', scale='utc', precision=6)
                except:
                    date = None  # FIXME: this might blow up, not sure
            else:
                try:
                    date = Time(date, format='jd', scale='utc', precision=6)
                except ValueError:
                    try:
                        date = Time(date, format='mpc', scale='utc', precision=6)
                    except ValueError:
                        date = Time(date, scale='utc', precision=6)  # see if it can guess
        if hasattr(self, 'time') and isinstance(self.time, Time):
            dt = self.time - date
            if math.fabs(dt.sec) < 10:
                return
        jd = ctypes.c_double(date.jd)
        if abg_file is None:
            abg_file = tempfile.NamedTemporaryFile(suffix='.abg')
            abg_file.write(self.abg)
            abg_file.seek(0)
        # call predict with agbfile, jdate, obscode
        self.orbfit.predict.restype = ctypes.POINTER(ctypes.c_double * 5)
        self.orbfit.predict.argtypes = [ctypes.c_char_p, ctypes.c_double, ctypes.c_int]
        predict = self.orbfit.predict(ctypes.c_char_p(abg_file.name),
                                      jd,
                                      ctypes.c_int(obs_code))
        self._coordinate = SkyCoord(predict.contents[0],
                                    predict.contents[1],
                                    unit=(units.degree, units.degree),
                                    obstime=date)
        self._dra = predict.contents[2] * units.arcsec
        self._ddec = predict.contents[3] * units.arcsec
        self._pa = predict.contents[4] * units.degree
        self._date = str(date)
        self._time = date

    def rate_of_motion(self, date=None):
        """
        The rate of motion of the date of the current predicted location, or the date given if specified.
        :param date: Time
        :return: rate of motion
        :rtype: Quantity
        """
        # rate of motion at a requested date rather than averaged over the arc.
        # Date is datetime.datetime() objects.
        if date is None:
            if self.time is None:
                date = datetime.datetime.now()
            else:
                date = self.time
        if date is None:
            return None
        if isinstance(date, datetime.datetime):
            try:
                date = date.strftime('%Y-%m-%d %H:%M:%S')
            except Exception as e:
                logging.error(str(e))
                return None
        if isinstance(date, str):
            date = Time(date, scale='utc')

        try:
            sdate = date.jd
            edate = date.jd + 1
        except Exception as e:
            logging.error(str(e))
            return None
        self.predict(edate)
        coord1 = self.coordinate
        self.predict(sdate)
        coord2 = self.coordinate
        return coord1.separation(coord2).to(units.arcsec) / (24. * units.hour)  # arcsec/hr

    def summarize(self, date=datetime.datetime.now()):
        """Return a string summary of the orbit.
        :rtype: str
        """

        assert isinstance(date, datetime.datetime)
        at_date = Time(date.strftime('%Y-%m-%d'), scale='utc')
        self.predict(at_date)

        fobj = StringIO()

        # for observation in self.observations:
        # fobj.write(observation.to_string()+"\n")

        fobj.write("\n")
        fobj.write(str(self) + "\n")
        fobj.write(str(self.residuals()) + "\n")
        fobj.write('arclen {} '.format(self.arc_length))
        fobj.write("Expected accuracy on {:>10s}: {:6.2f} {:6.2f} moving at {:6.2f} \n\n".format(
            at_date, self.dra, self.ddec, self.rate_of_motion(date=date)))

        fobj.seek(0)
        return fobj.read()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx

from ossos.gui import config, logger


class KeybindManager(object):
    def __init__(self, view, controller):
        """

        :param view:
        :param controller: the controller to bind this key to
        :type controller: AbstractController
        :return:
        """
        logger.debug("Building KeybindManager.")
        self.controller = controller
        self.view = view

        next_obs_kb_id = wx.NewId()
        prev_obs_kb_id = wx.NewId()
        accept_src_kb_id = wx.NewId()
        auto_accept_src_kd_id = wx.NewId()
        reject_src_kb_id = wx.NewId()
        reset_cmap_kb_id = wx.NewId()
        reset_src_kb_id = wx.NewId()
        autoplay_kb_id = wx.NewId()
        load_comparison_kb_id = wx.NewId()
        load_diff_comparison_kb_id = wx.NewId()
        toggle_reticule_kb_id = wx.NewId()
        toggle_align_kb_id = wx.NewId()

        def bind(handler, kb_id):
            view.Bind(wx.EVT_MENU, handler, id=kb_id)

        bind(self.on_load_comparison_keybind, load_comparison_kb_id)
        bind(self.on_load_diff_comparison_keybind, load_diff_comparison_kb_id)
        bind(self.on_next_obs_keybind, next_obs_kb_id)
        bind(self.on_prev_obs_keybind, prev_obs_kb_id)
        bind(self.on_accept_src_keybind, accept_src_kb_id)
        bind(self.on_auto_accept_src_keybind, auto_accept_src_kd_id)
        bind(self.on_reject_src_keybind, reject_src_kb_id)
        bind(self.on_reset_cmap_keybind, reset_cmap_kb_id)
        bind(self.on_reset_source_location_keybind, reset_src_kb_id)
        bind(self.on_toggle_autoplay, autoplay_kb_id)
        bind(self.on_toggle_reticule, toggle_reticule_kb_id)
        bind(self.on_toggle_align, toggle_align_kb_id)

        self.toggle_align_key = config.read("KEYBINDS.TOGGLE_ALIGN")
        self.accept_key = config.read("KEYBINDS.ACCEPT_SRC")
        self.auto_accept_key = config.read("KEYBINDS.AUTO_ACCEPT_SRC")
        self.reject_key = config.read("KEYBINDS.REJECT_SRC")
        self.reset_cmap_key = config.read("KEYBINDS.RESET_CMAP")
        self.reset_source_key = config.read("KEYBINDS.RESET_SOURCE_LOCATION")
        self.autoplay_key = config.read("KEYBINDS.AUTOPLAY")
        self.toggle_reticule_key = config.read("KEYBINDS.TOGGLE_RETICULE")
        self.load_comparison_key = config.read("KEYBINDS.LOAD_COMPARISON")
        self.load_diff_comparison_key = config.read("KEYBINDS.LOAD_DIFF_COMPARISON")

        accelerators = [
            (wx.ACCEL_NORMAL, wx.WXK_TAB, next_obs_kb_id),
            (wx.ACCEL_SHIFT, wx.WXK_TAB, prev_obs_kb_id),
            (wx.ACCEL_NORMAL, ord(self.accept_key), accept_src_kb_id),
            (wx.ACCEL_NORMAL, ord(self.auto_accept_key), auto_accept_src_kd_id),
            (wx.ACCEL_NORMAL, ord(self.reject_key), reject_src_kb_id),
            (wx.ACCEL_NORMAL, ord(self.reset_cmap_key), reset_cmap_kb_id),
            (wx.ACCEL_NORMAL, ord(self.reset_source_key), reset_src_kb_id),
            (wx.ACCEL_NORMAL, ord(self.load_comparison_key), load_comparison_kb_id),
            (wx.ACCEL_NORMAL, ord(self.load_diff_comparison_key), load_diff_comparison_kb_id),
            (wx.ACCEL_NORMAL, ord(self.autoplay_key), autoplay_kb_id),
            (wx.ACCEL_NORMAL, ord(self.toggle_reticule_key), toggle_reticule_kb_id),
            (wx.ACCEL_NORMAL, ord(self.toggle_align_key), toggle_align_kb_id),
        ]

        for line in config.read("MPC.NOTE1OPTIONS"):
            if not len(line) > 0:
                continue
            this_key = line.split()[0][0].lower()
            this_func = self._build_accept_func(this_key)
            this_key_id = wx.NewId()
            bind(this_func, this_key_id)
            accelerators.append((wx.ACCEL_CTRL, ord(this_key), this_key_id))

        accelerators = wx.AcceleratorTable(accelerators)
        view.SetAcceleratorTable(accelerators)

    def _build_accept_func(self, key):
        def func(event):
            self.controller.on_accept(auto=key)
        return func

    def get_keymappings(self):
        return [("Next observation", "Tab"),
                ("Previous observation", "Shift + Tab"),
                ("Accept", self.accept_key),
                ("Auto Accept", self.auto_accept_key),
                ("Reject", self.reject_key),
                ("Reset colourmap", self.reset_cmap_key),
                ("Reset source location", self.reset_source_key),
                ("Autoplay", self.autoplay_key),
                ("Load Comparison Image", self.load_comparison_key),
                ("Load Difference Comaprison", self.load_diff_comparison_key),
                ("Toggle reticule", self.toggle_reticule_key),
                ("Toggle focus", self.toggle_align_key)]

    def on_load_diff_comparison_keybind(self, event):
        logger.debug("BBB!")
        self.controller.on_load_comparison(research=True)

    def on_load_comparison_keybind(self, event):
        self.controller.on_load_comparison()

    def on_next_obs_keybind(self, event):
        self.controller.on_next_obs()

        # Note: event consumed (no call to event.Skip()) so that we don't
        # have tab iterating over the buttons like it does by default

    def on_prev_obs_keybind(self, event):
        self.controller.on_previous_obs()

    def on_auto_accept_src_keybind(self, event):
        self.controller.on_accept(auto=True)

    def on_accept_src_keybind(self, event):
        self.controller.on_accept()

    def on_reject_src_keybind(self, event):
        self.controller.on_reject()

    def on_reset_cmap_keybind(self, event):
        self.controller.on_reset_colormap()

    def on_reset_source_location_keybind(self, event):
        self.controller.on_reset_source_location()

    def on_toggle_autoplay(self, event):
        self.controller.on_toggle_autoplay_key()

    def on_toggle_reticule(self, event):
        self.controller.on_toggle_reticule_key()

    def on_toggle_align(self, event):
        self.controller.on_toggle_align()



__author__ = 'Michele Bannister   git:@mtbannister'

import matplotlib.pyplot as plt
import matplotlib
import mpld3
from mpld3 import plugins

import parsers
import plot_fanciness


matplotlib.rcParams['font.size'] = 20  # good for posters/slides
matplotlib.rcParams['font.family'] = 'sans-serif'
matplotlib.rcParams['font.sans-serif'] = ['Sofia Pro', 'Tahoma']


# Define some CSS to control our custom labels
css = """
table
{
  border-collapse: collapse;
}
th
{
  color: #ffffff;
  background-color: #000000;
}
td
{
  background-color: #cccccc;
}
table, th, td
{
  font-family:Sofia Pro, sans-serif;
  border: 1px solid black;
  text-align: right;
}
"""

data = parsers.ossos_release_with_metadata()
# Just plot the sensible ones for now
dset = [d for d in data if ((d.a is not None) and (d.a_e / d.a < 0.05))]

# # suitably bright ones
# bright = [d for d in data if d.mag < 23.5]
# print len(bright)
#
# # just pulling out the cold classicals...
# temp = [d for d in data if (42. < d.a < 46.) and d.e < 0.15 and d.i < 5. and d.mag < 23.5]
# print len(temp)
# data = temp

fig, ax = plt.subplots(2, 1, sharex=True, figsize=(10, 10))
fig.subplots_adjust(hspace=0.05)

# why is this one object not fitting at all correctly?
# and not d.observations[-1].provisional_name.strip(' ').__contains__('o3e17'))]
# for n in dset:
# if n.observations[-1].provisional_name.strip(' ').__contains__('o3e38'):
#         print n.a, n.inc, n.e

# For calling with parsers.ossos_release_with_metadata()
points = ax[0].errorbar([d.a for d in dset], [d.i for d in dset],
                        xerr=[d.a_e for d in dset], yerr=[d.i_e for d in dset],
                        fmt='.', alpha=0.4, ecolor='0.1', ms=10)

# For using Orbfit objects
# points = ax[0].errorbar([d.a for d in dset], [d.inc for d in dset],
#                         xerr=[d.da for d in dset], yerr=[d.dinc for d in dset],
#                         fmt='.', alpha=0.4, ecolor='0.1', ms=15)

ax[0].set_ylabel('inclination (degrees)')
ax[0].set_ylim([-1, 55])
ax[0].grid(True, alpha=0.3)

# For calling with parsers.ossos_release_with_metadata()
ax[1].errorbar([d.a for d in dset], [d.e for d in dset],
               xerr=[d.a_e for d in dset], yerr=[d.e_e for d in dset],
               fmt='.', alpha=0.4, ecolor='0.1', ms=10)

# For using Orbfit objects
# ax[1].errorbar([d.a for d in dset], [d.e for d in dset],
#                xerr=[d.da for d in dset], yerr=[d.de for d in dset],
#                fmt='.', alpha=0.4, ecolor='0.1', ms=15)

ax[1].set_ylabel('eccentricity')
ax[1].set_ylim([-0.01, .8])
ax[1].grid(True, alpha=0.3)
# ax[1].set_xlabel('semimajor axis (AU)')
plt.xlim([20, 155])

plot_fanciness.remove_border(ax[0])
plot_fanciness.remove_border(ax[1])

plt.xlabel('semimajor axis (AU)')

# plt.legend()
plt.draw()
outfile = 'Sep24_OSSOS_discoveries_orbits'
plt.savefig(outfile + '.pdf', transparent=True)

# plugins.connect(fig, LinkedBrush(points))

labels = []
for obj in data:
    peri = None
    try:
        peri = obj.peri
    except:
        peri = obj.a * (1. - obj.e)
        obj.peri = peri
    if peri is not None:
        # label = {'dist': obj.dist, 'peri':obj.peri, 'obj':obj.name}
        html = '<table><tr><th>Name</th><td>{:}</td></tr>' \
               '<tr><th>Dist</th><td>{:.2f}</td></tr>' \
               '<tr><th>Peri</th><td>{:.2f}</td></tr></table>'.format(obj.name, obj.dist, peri)
        labels.append(html)

# Activate the interactivity
tooltip = plugins.PointHTMLTooltip(points[0], labels, voffset=10, hoffset=10, css=css)
plugins.connect(fig, tooltip)
mpld3.show()


# Generate a new plot of the objects designed to match the parameters in Petit et al. 2011
# layout is to be top left, a-q, top right, i-q, lower left, a-i.
# For calling with parsers.ossos_release_with_metadata()

plt.close()
fig, ax = plt.subplots(2, 2, figsize=(10, 10))
fig.subplots_adjust(hspace=0.0)  # want to work on the v-space as well

points = ax[0][0].errorbar([d.a for d in dset], [d.peri for d in dset],
                           xerr=[d.a_e for d in dset],
                           fmt='.', alpha=0.4, ecolor='0.1', ms=10)
ax[0][0].grid(True, alpha=0.4)
ax[0][0].set_xlim([39., 48.])
ax[1][0].set_xticks(range(40, 49, 2))
ax[0][0].set_ylim([34., 48.])
ax[0][0].set_ylabel('pericenter (AU)')
ax[0][0].set_yticks(range(36, 50, 4))

ax[0][1].errorbar([d.i for d in dset], [d.peri for d in dset],
                  xerr=[d.i_e for d in dset],
                  fmt='.', alpha=0.4, ecolor='0.1', ms=10)
ax[0][1].grid(True, alpha=0.4)
ax[0][1].set_xlim([0., 35.])
ax[0][1].set_xlabel('inclination (deg)')
ax[0][1].set_xticks(range(5, 40, 10))
ax[0][1].set_ylim([34., 48.])
ax[0][1].set_yticks(range(36, 50, 4))

ax[1][0].errorbar([d.a for d in dset], [d.i for d in dset],
                  xerr=[d.a_e for d in dset], yerr=[d.i_e for d in dset],
                  fmt='.', alpha=0.4, ecolor='0.1', ms=10)
ax[1][0].set_xlim([39., 48.])
ax[1][0].set_xticks(range(40, 49, 2))
ax[1][0].set_xlabel('semimajor axis (AU)')
ax[1][0].set_ylim([0., 35.])
ax[1][0].set_ylabel('inclination (deg)')
ax[1][0].set_yticks(range(0, 40, 10))
ax[1][0].grid(True, which='both', alpha=0.4)

plot_fanciness.remove_border(ax[0][0])
ax[0][0].tick_params(labelbottom='off')
plot_fanciness.remove_border(ax[0][1])
plot_fanciness.remove_border(ax[1][0])
plot_fanciness.remove_border(ax[1][1], remove=['left', 'right', 'top', 'bottom'], keep=[])
ax[1][1].tick_params(labelbottom='off', labelleft='off')

plt.draw()
outfile = 'Sep24_OSSOS_discoveries_Petit_style'
plt.savefig(outfile + '.pdf', transparent=True)




# # now plot again with the tno sizes indicated (approximating albedo to 4%)
#
# fig, ax = plt.subplots(3, 1, sharex=True, figsize=(10,10))
# ax[0].set_title('OSSOS discoveries in v2 release with points scaled by object size')
# fig.subplots_adjust(hspace=0.12)
#
# for p in data:
#     #diameter = ( (p.dist**2)*(10**((18.8 - p.mag)/5.)) )/math.sqrt(0.04)  # Fraser and Kavelaars, 2008
#     au = 149597871.
#     K = 5.61 # Vilenius et al., 2012
#     albedo = 0.1 # reasonable interval between cold and hot, could subdivide later as desired by incl.
#     diameter = 10**(math.sqrt((5.*math.log10((p.dist*au)**2) + K - p.mag)/2.5*albedo))
#
#     print p.dist, diameter, diameter*100., p.mag
#     p.diameter = diameter
#
# # first all the objects. Some of these don't have all their orbital parameters yet, but dist and i are ok
# for p in data:  # do this point-by-point to calculate individual object sizes
#     ax[0].errorbar(p.dist, p.i,
#                    xerr=p.dist_e, yerr=p.i_e,
#                    fmt='.', alpha=0.3, ecolor='0.1', ms=p.diameter/100., mfc='b', mec='b')
# ax[0].set_ylim([-1, 90])
# ax[0].grid(True, alpha=0.3)
# ax[0].set_ylabel('inclination (degrees)')
# ax[0].set_xlabel('heliocentric distance (AU)')
# # ax[0].annotate('')
#
# # now the plots for objects where we have much better arcs
# dset = [d for d in data if d.a is not None]
# for p in dset:
#     ax[1].errorbar(p.a, p.i,
#                     xerr=p.a_e, yerr=p.i_e,
#                     fmt='.', alpha=0.4, ecolor='0.1', ms=p.diameter/100., mfc='b', mec='b')
# ax[1].set_ylabel('inclination (degrees)')
# ax[1].set_ylim([-1, 35])
# ax[1].grid(True, alpha=0.3)
# ax[1].set_xlabel('semimajor axis (AU)')
#
# for p in dset:
#     ax[2].errorbar(p.a, p.e,
#                    xerr=p.a_e, yerr=p.e_e,
#                    fmt='.', alpha=0.4, ecolor='0.1', ms=p.diameter/100., mfc='b', mec='b')
# ax[2].set_ylabel('eccentricity')
# ax[2].set_ylim([-0.01, .8])
# ax[2].grid(True, alpha=0.3)
# ax[2].set_xlabel('semimajor axis (AU)')
#
# # plt.show()
#
# plt.draw()
# outfile = 'plots/2013_OSSOS_discoveries_sizes'
# plt.savefig(outfile+'.pdf', transparent=True)



__author__ = "David Rusk <drusk@uvic.ca>"

import wx

from .. import logger


class Menu(object):
    def __init__(self, frame, controller):
        logger.debug("Building {}.".format(self))
        self.controller = controller
        self.frame = frame

        self._build_menu()

    def _build_menu(self):
        def do_bind(handler, item):
            self.frame.Bind(wx.EVT_MENU, handler, item)

        # Create menus and their contents
        file_menu = wx.Menu()
        keymap_item = file_menu.Append(
            id=wx.ID_ANY,
            text="Keymap",
            help="Show mappings for keyboard shortcuts.")
        exit_item = file_menu.Append(
            id=wx.ID_EXIT,
            text="Exit",
            help="Exit the program")

        do_bind(self._on_select_keymap, keymap_item)
        do_bind(self._on_select_exit, exit_item)

        display_menu = wx.Menu()
        singlet_view_item = display_menu.Append(
            id=wx.ID_ANY,
            text="Single images",
            kind=wx.ITEM_RADIO)
        triplet_view_item = display_menu.Append(
            id=wx.ID_ANY,
            text="Image triplets",
            kind=wx.ITEM_RADIO)
        display_menu.AppendSeparator()
        auto_play_item = display_menu.Append(
            id=wx.ID_ANY,
            text="Autoplay",
            help="Automatically transition through images.",
            kind=wx.ITEM_CHECK)

        do_bind(self._on_select_singlet_view, singlet_view_item)
        do_bind(self._on_select_triplet_view, triplet_view_item)
        do_bind(self._on_select_autoplay, auto_play_item)

        sync_menu = wx.Menu()
        auto_sync_item = sync_menu.Append(
            id=wx.ID_ANY,
            text="Automatically",
            help="Automatically synchronize results.",
            kind=wx.ITEM_CHECK)

        do_bind(self._on_select_automatically_sync, auto_sync_item)

        # Create menu bar
        menubar = wx.MenuBar()
        self.file_menu_title = "File"
        menubar.Append(file_menu, self.file_menu_title)
        self.display_menu_title = "Display"
        menubar.Append(display_menu, self.display_menu_title)
        self.sync_menu_title = "Sync"
        menubar.Append(sync_menu, self.sync_menu_title)

        self.frame.SetMenuBar(menubar)

        self.menubar = menubar
        self.auto_play_item = auto_play_item

    def _on_select_keymap(self, event):
        self.controller.on_show_keymappings()

    def _on_select_exit(self, event):
        self.controller.on_exit()

    def _on_select_automatically_sync(self, event):
        if event.Checked():
            self.controller.on_enable_auto_sync()
        else:
            self.controller.on_disable_auto_sync()

    def _on_select_singlet_view(self, event):
        self.controller.on_use_singlet_view()

    def _on_select_triplet_view(self, event):
        self.controller.on_use_triplet_view()

    def _on_select_autoplay(self, event):
        if event.Checked():
            self.controller.on_enable_autoplay()
        else:
            self.controller.on_disable_autoplay()

    def disable_menu_item(self, item_title):
        self.menubar.EnableTop(
            self.menubar.FindMenu(item_title), False)

    def disable_sync(self):
        self.disable_menu_item(self.sync_menu_title)

    def set_autoplay(self, autoplay_enabled):
        self.auto_play_item.Check(autoplay_enabled)


from ossos import mpc, storage, orbfit
from ossos import wcs
import sys
import numpy
from astropy.table import Table, Column


def ephem_search(mpc_filename, search_date="2014 07 24.0"):
    """
    builds a TSV file in the format of SSOIS by querying for possible observations in CADC/CAOM2.

    This is a fall back program, should only be useful when SSOIS is behind.
    """
    columns = ('Image',
               'Ext',
               'X',
                'Y',
                'MJD',
                'Filter',
                'Exptime',
                'Object_RA',
                'Object_Dec',
                'Image_target',
                'Telescope/Instrument',
                'MetaData',
                'Datalink')

    ephem_table = Table(names=columns,
                         dtypes=('S10', 'i4', 'f8', 'f8',
                                 'f8', 'S10', 'f8', 'f8', 'f8', 'S20', 'S20', 'S20', 'S50'))

    ephem_table.pprint()

    o = orbfit.Orbfit(mpc. MPCReader(mpc_filename).mpc_observations)
    o.predict(search_date)
    fields = storage.cone_search(o.coordinate.ra.degrees, o.coordinate.dec.degrees, dra=0.3, ddec=0.3,
                                 calibration_level=1)
    mjdates = numpy.unique(fields['mjdate'])

    collectionIDs = []
    for mjdate in mjdates:
        jd = 2400000.5 + mjdate
        o.predict(jd)
        for field in storage.cone_search(o.coordinate.ra.degrees, o.coordinate.dec.degrees,
                                         dra=30./3600.0, ddec=30./3600.0,
                                         mjdate=mjdate, calibration_level=1):
            collectionIDs.append(field['collectionID'])

    expnums = numpy.unique(numpy.array(collectionIDs))

    for expnum in expnums:
        header = storage.get_astheader(expnum, 22)
        o.predict(header['MJDATE']+2400000.5)
        print o.time.iso, o.coordinate.ra.degrees, o.coordinate.dec.degrees
        for ccd in range(36):
            header = storage.get_astheader(expnum, ccd)
            w = wcs.WCS(header)
            (x, y) = w.sky2xy(o.coordinate.ra.degrees, o.coordinate.dec.degrees)
            print ccd, x, y
            if 0 < x < header['NAXIS1'] and 0 < y < header['NAXIS2']:
                ephem_table.add_row([expnum, ccd+1, x, y,
                                     header['MJDATE'], header['FILTER'], header['EXPTIME'],
                                     o.coordinate.ra.degrees, o.coordinate.dec.degrees,
                                     header['OBJECT'],
                                     'CFHT/MegaCam',
                                     None,
                                     "http://www.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/data/pub/CFHT/{}p[{}]".format(expnum, ccd)])
                break

    ephem_table.pprint()
    ephem_table.write('backdoor.tsv', format='ascii', delimiter='\t')


if __name__ == '__main__':
    ephem_search(sys.argv[0])



import re
import sys

import ephem


#Number_Mil={'B': 110000, 'C': 120000, 'D': 130000, 'E': 140000, 'F': 150000}
#Number_Cent={'J': 1900, 'K': 2000}
def date_unpack(pdate):
    (yyyy, mm, dd) = (2000, 01, 01)
    try:
        YY={'I': 1800, 'J': 1900, 'K': 2000}
        Ncode='0123456789ABCDEFGHIJKLMNOPQRSTUV'
        yyyy=YY[pdate[0]]+int(pdate[1:3])
        mm=Ncode.rindex(pdate[3])
        dd=float(Ncode.rindex(pdate[4]))
    except:
        sys.stderr.write("ERROR converting date part {}:".format(pdate))
    return (yyyy, mm, dd)


def desig_unpack(desig):
    if re.match('\d+',desig):
        return str(int(desig))
    Kilo='ABCDEFGHIJKLMNOPQRSTUV'
    j = Kilo.rfind(desig[0])
    if j != -1 :
      if re.match('^\d+$',desig[1:7]):
        return str(100000+j*10000+int(desig[1:7]))
    YY={'I': 1800, 'J': 1900, 'K': 2000}
    try:
       yyyy=str(YY[desig[0]]+int(desig[1:3]))
    except KeyError as e:
       return desig
    Ncode='0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
    Mcode=desig[3]+desig[6]
    cycle=Ncode.rindex(desig[4])*10+int(desig[5])
    if (cycle>0) :
	cycle=str(cycle)
    else:
	cycle=''
    return yyyy+' '+Mcode+cycle
    
def getKBOs(mpc_file, cond='a > 30'):

    f=open(mpc_file)
    lines=f.readlines()
    f.close()

    # while "------" not in lines[0]:
    # lines.pop(0)
    # lines.pop(0)
    nobj=0
    lineCount=0
    kbos = []
    for line in lines:
        lineCount=lineCount+1
        if line[0] == '#' or len(line) < 103 or line[0:3] == '---' or line[0:3] == 'Des':
            continue
        kbo=ephem.EllipticalBody()

        try:
            if len(line[8:13].strip()):
                kbo._H = float(line[8:13])
                kbo._G = float(line[14:19])
            else:
                kbo._H = 20
                kbo._G = 0
        except:
            print line
            kbo._H = 20
            kbo._G = 0
        arc = line[127:136]
        try:
            if 'days' in arc:
                arc = int(arc.split()[0])/365.25
            else:  
                arc = -eval(arc)
        except:
            arc = 0
        kbo._epoch_M=date_unpack(line[20:25].strip())
        kbo._M=float(line[26:35].strip())
        kbo._om=float(line[37:46].strip())
        kbo._Om=float(line[48:57].strip())
        kbo._inc=float(line[59:68].strip())
        kbo._e=float(line[70:79].strip())
        kbo._epoch='2011/08/01'
        kbo._a=float(line[92:103].strip())
        kbo.compute()

        a = kbo._a
        q = kbo._a*(1-kbo._e)
        H = kbo._H

        if eval(cond): 
            kbo.name=desig_unpack(line[0:7].strip()) 
            kbos.append(kbo)

    return kbos



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to

from tests.matchers import round_sigfigs, almost_equal


class MatchersTest(unittest.TestCase):
    def test_round_sigfigs(self):
        assert_that(round_sigfigs(1234, 3), equal_to(1230.0))

    def test_round_sigfigs_neg(self):
        assert_that(round_sigfigs(-1234, 3), equal_to(-1230.0))

    def test_round_sigfigs_more_than_input(self):
        assert_that(round_sigfigs(1234, 7), equal_to(1234.000))

    def test_assert_almost_equal(self):
        assert_that(1.234, almost_equal(1.23444, 4))

    def test_assert_almost_equal2(self):
        assert_that(1.234012345, almost_equal(1.23403876, 5))
        # NOTE: not 5 sig figs because of the rounding.  Is this how it should be?
        assert_that(1.234012345, almost_equal(1.23409876, 4))


if __name__ == '__main__':
    unittest.main()



"""
Base classes for test cases.
"""

__author__ = "David Rusk <drusk@uvic.ca>"

import os
import unittest

import wx
from mock import Mock

from ossos.gui import events


class FileReadingTestCase(unittest.TestCase):
    def get_abs_path(self, relative_path):
        """
        Get the absolute path of a file in the test directory.

        Args:
          relative_path: str
            path of the file with respect to the base of the test directory
            ex: data/datafile.csv

        Returns:
          absolute_path: str
            The absolute path to the file.
        """
        if relative_path.startswith("/"):
            relative_path = relative_path[1:]
        relative_path.replace("/", os.sep)
        return os.path.join(os.path.dirname(__file__), relative_path)


class WxWidgetTestCase(unittest.TestCase):
    def setUp(self):
        events.unsub_all()
        self.app = wx.App(False)
        self.rootframe = wx.Frame(None)

    def tearDown(self):
        self.rootframe.Destroy()
        self.app.Exit()

    def _get_by_equality(self, iterable, is_equal):
        for item in iterable:
            if is_equal(item):
                return item
        return None

    def _get_by_label(self, label, iterable):
        def has_same_label(item):
            return item.GetLabel().lower() == label.lower()

        return self._get_by_equality(iterable, has_same_label)

    def assert_has_child_with_label(self, parent, label):
        self.assertIsNotNone(self.get_child_by_label(parent, label))

    def get_child_by_label(self, parent, label):
        return self._get_by_label(label, parent.GetChildren())

    def get_child_by_name(self, parent, name):
        def has_same_name(item):
            return item.GetName() == name

        return self._get_by_equality(parent.GetChildren(), has_same_name)

    def get_menuitem_by_label(self, menu, label):
        return self._get_by_label(label, menu.GetMenuItems())

    def fire_button_click_event(self, widget):
        event = wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED,
                                widget.GetId())
        widget.GetEventHandler().ProcessEvent(event)

    def mock_model(self):
        self.model = Mock()
        self.model.get_reading_data.return_value = (("TEST1", 1), ("TEST2", 2))
        self.model.get_header_data_list.return_value = [("Key1", "Val1"), ("Key2", "Val2")]
        self.model.get_current_image_source_point.return_value = (0, 0)
        self.model.get_current_image_FWHM.return_value = 3
        self.model.get_current_source_number.return_value = 0
        self.model.get_source_count.return_value = 2
        self.model.get_current_obs_number.return_value = 0
        self.model.get_obs_count.return_value = 3


class DirectoryCleaningTestCase(unittest.TestCase):
    def tearDown(self):
        directory = self.get_directory_to_clean()
        for filename in os.listdir(directory):
            if filename not in self.get_files_to_keep():
                os.remove(os.path.join(directory, filename))

    def get_directory_to_clean(self):
        raise NotImplementedError()

    def get_files_to_keep(self):
        raise NotImplementedError()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx


class MPCPanel(wx.Panel):
    def __init__(self, parent, controller,
                 ssos_label="Query", save_label="Save"):
        super(MPCPanel, self).__init__(parent)

        self.controller = controller

        self.ssos_label = ssos_label
        self.save_label = save_label

        self._init_ui()

    def _init_ui(self):
        self.sbox = wx.StaticBox(self, label="MPC Builder")

        self.ssos_button = wx.Button(self, label=self.ssos_label)
        self.save_button = wx.Button(self, label=self.save_label)

        self.ssos_button.Bind(wx.EVT_BUTTON, self._on_click_ssos)
        self.save_button.Bind(wx.EVT_BUTTON, self._on_click_save)

        self._do_layout()

    def _do_layout(self):
        sbox_sizer = wx.StaticBoxSizer(self.sbox, wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        button_border = 10
        hsizer.Add(self.save_button, proportion=0, flag=wx.ALL, border=button_border)
        hsizer.Add(self.ssos_button, proportion=0, flag=wx.ALL, border=button_border)

        sbox_sizer.Add(hsizer, flag=wx.ALIGN_CENTER)

        # Add a bit of border around the box sizer
        border_sizer = wx.BoxSizer(wx.VERTICAL)
        border_sizer.Add(sbox_sizer, flag=wx.EXPAND | wx.ALL, border=10)

        self.SetSizer(border_sizer)

    def _on_click_ssos(self, event):
        self.controller.on_ssos()

    def _on_click_save(self, event):
        self.controller.on_save()

    def disable(self):
        self.save_button.Disable()
        self.ssos_button.Disable()

    def enable(self):
        self.save_button.Enable()
        self.ssos_button.Enable()

    def is_validation_enabled(self):
        return self.save_button.IsEnabled() and self.ssos_button.IsEnabled()


class SourceValidationPanel(wx.Panel):
    def __init__(self, parent, controller,
                 accept_label="Accept", reject_label="Reject"):
        super(SourceValidationPanel, self).__init__(parent)

        self.controller = controller

        self.accept_label = accept_label
        self.reject_label = reject_label

        self._init_ui()

    def _init_ui(self):
        self.sbox = wx.StaticBox(self, label="Source Validation")

        self.accept_button = wx.Button(self, label=self.accept_label)
        self.reject_button = wx.Button(self, label=self.reject_label)

        self.accept_button.Bind(wx.EVT_BUTTON, self._on_click_accept)
        self.reject_button.Bind(wx.EVT_BUTTON, self._on_click_reject)

        self._do_layout()

    def _do_layout(self):
        sbox_sizer = wx.StaticBoxSizer(self.sbox, wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        button_border = 10
        hsizer.Add(self.accept_button, proportion=0, flag=wx.ALL, border=button_border)
        hsizer.Add(self.reject_button, proportion=0, flag=wx.ALL, border=button_border)

        sbox_sizer.Add(hsizer, flag=wx.ALIGN_CENTER)

        # Add a bit of border around the box sizer
        border_sizer = wx.BoxSizer(wx.VERTICAL)
        border_sizer.Add(sbox_sizer, flag=wx.EXPAND | wx.ALL, border=10)

        self.SetSizer(border_sizer)

    def _on_click_accept(self, event):
        self.controller.on_accept()

    def _on_click_reject(self, event):
        self.controller.on_reject()

    def disable(self):
        self.accept_button.Disable()
        self.reject_button.Disable()

    def enable(self):
        self.accept_button.Enable()
        self.reject_button.Enable()

    def is_validation_enabled(self):
        return self.accept_button.IsEnabled() and self.reject_button.IsEnabled()


class SourceValidationDialog(wx.Dialog):
    COMMENT = "Comment: "
    SUBMIT_BTN = "Submit"
    CANCEL_BTN = "Cancel"

    def __init__(self, parent, title=""):
        super(SourceValidationDialog, self).__init__(parent, title=title)

        self._init_ui()

        self.comment_label = wx.StaticText(self, label=SourceValidationDialog.COMMENT)
        self.comment_text = wx.TextCtrl(self, name=SourceValidationDialog.COMMENT)

        #self.comment_text.SetValue(self.default_comment)
        #self.comment_text.Bind(wx.EVT_TEXT_ENTER, self._on_enter_comment)

        self.submit_button = wx.Button(
            self, label=self.SUBMIT_BTN, name=SourceValidationDialog.SUBMIT_BTN)
        self.cancel_button = wx.Button(
            self, label=self.CANCEL_BTN, name=SourceValidationDialog.CANCEL_BTN)

        self.submit_button.Bind(wx.EVT_BUTTON, self._on_submit)
        self.cancel_button.Bind(wx.EVT_BUTTON, self._on_cancel)

        self._do_layout()

        self.submit_button.SetDefault()

    def _create_horizontal_pair(self, widget1, widget2, flag=0, border=0):
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(widget1, flag=flag, border=border)
        hsizer.Add(widget2, flag=flag, border=border)
        return hsizer

    def _do_layout(self):
        vsizer = wx.BoxSizer(wx.VERTICAL)
        for widget in self._get_vertical_widget_list():
            vsizer.Add(widget, proportion=0, flag=wx.ALL | wx.EXPAND, border=5)

        comment_sizer = wx.BoxSizer(wx.VERTICAL)
        comment_sizer.Add(self.comment_label, flag=wx.ALIGN_CENTER)
        comment_sizer.Add(self.comment_text, flag=wx.EXPAND)

        vsizer.Add(comment_sizer, flag=wx.ALL | wx.EXPAND, border=5)
        vsizer.Add(self._create_horizontal_pair(self.submit_button, self.cancel_button,
                                                flag=wx.ALL, border=5),
                   flag=wx.ALIGN_CENTER)

        # Extra border padding
        bordersizer = wx.BoxSizer(wx.VERTICAL)
        bordersizer.Add(vsizer, flag=wx.ALL, border=20)

        self.SetSizerAndFit(bordersizer)

    def _on_enter_comment(self, event):
        # Don't want it to actually put a newline character, just submit
        # the form.
        self._on_submit(event)

    def _init_ui(self):
        raise NotImplementedError()

    def _on_submit(self, event):
        raise NotImplementedError()

    def _on_cancel(self, event):
        raise NotImplementedError()

    def _get_vertical_widget_list(self):
        raise NotImplementedError()


class AcceptSourceDialog(SourceValidationDialog):
    TITLE = "Accept Source"
    MINOR_PLANET_NUMBER = "Minor planet number: "
    PROVISIONAL_NAME = "Provisional name: "
    NOTE1 = "Note 1: "
    NOTE2 = "Note 2: "
    DATE_OF_OBS = "Date of observation: "
    RA = "Right ascension: "
    DEC = "Declination: "
    OBS_MAG = "Observed magnitude: "
    BAND = "Band: "
    OBSERVATORY_CODE = "Observatory code: "

    def __init__(self, parent, controller,
                 provisional_name,
                 date_of_obs,
                 ra,
                 dec,
                 obs_mag,
                 obs_mag_err,
                 band,
                 note1_choices=None,
                 note2_choices=None,
                 note1_default=None,
                 note2_default=None,
                 default_observatory_code="",
                 default_comment="",
                 phot_failure=False,
                 pixel_x=None,
                 pixel_y=None):
        self.controller = controller
        self.phot_failure = phot_failure

        self.provisional_name = provisional_name
        self.date_of_obs = date_of_obs
        self.ra = ra
        self.dec = dec
        self.ra_str = "{:>14} {:>5.2f}".format(ra, float(pixel_x))
        self.dec_str = "{:>14} {:>5.2f}".format(dec, float(pixel_y))

        if self.phot_failure:
            message = "Photometry failed.  Will be left blank."
            self.obs_mag = message
            self.band = message
            self.obs_mag_err = message
        else:
            self.obs_mag = "{:5.2f}".format(obs_mag)
            self.obs_mag_err = "{:5.2f}".format(obs_mag_err)
            self.band = "{}".format(band)

        self.default_observatory_code = str(default_observatory_code)

        self.note1_choices = note1_choices if note1_choices is not None else []
        self.note2_choices = note2_choices if note2_choices is not None else []

        self.note1_default = note1_default if note1_default is not None else ""
        self.note2_default = note2_default if note2_default is not None else ""

        self.default_comment = default_comment

        super(AcceptSourceDialog, self).__init__(parent, title=self.TITLE)

        self.note1_combobox.SetFocus()

    def _init_ui(self):
        self.minor_planet_num_label = wx.StaticText(
            self, label=AcceptSourceDialog.MINOR_PLANET_NUMBER)
        self.minor_planet_num_text = wx.TextCtrl(
            self, name=AcceptSourceDialog.MINOR_PLANET_NUMBER)

        self.provisional_name_label = wx.StaticText(
            self, label=AcceptSourceDialog.PROVISIONAL_NAME)
        self.provision_name_text = self._create_readonly_text(
            value=self.provisional_name, name=self.PROVISIONAL_NAME)

        self.note1_label = wx.StaticText(self, label=AcceptSourceDialog.NOTE1)
        self.note1_combobox = KeyboardCompleteComboBox(
            self, value=self.note1_default, choices=self.note1_choices,
            name=AcceptSourceDialog.NOTE1)

        self.note2_label = wx.StaticText(self, label=AcceptSourceDialog.NOTE2)
        self.note2_combobox = KeyboardCompleteComboBox(
            self, value=self.note2_default, choices=self.note2_choices,
            name=AcceptSourceDialog.NOTE2)

        self.date_of_obs_label = wx.StaticText(
            self, label=AcceptSourceDialog.DATE_OF_OBS)
        self.date_of_obs_text = self._create_readonly_text(
            value=self.date_of_obs, name=AcceptSourceDialog.DATE_OF_OBS)

        self.ra_label = wx.StaticText(self, label=AcceptSourceDialog.RA)
        self.ra_text = self._create_readonly_text(
            value=self.ra_str, name=AcceptSourceDialog.RA)

        self.dec_label = wx.StaticText(self, label=AcceptSourceDialog.DEC)
        self.dec_text = self._create_readonly_text(
            value=self.dec_str, name=AcceptSourceDialog.DEC)

        self.obs_mag_label = wx.StaticText(self, label=AcceptSourceDialog.OBS_MAG)
        self.obs_mag_text = self._create_readonly_text(
            value=self.obs_mag, name=self.OBS_MAG)

        self.band_label = wx.StaticText(self, label=AcceptSourceDialog.BAND)
        self.band_text = self._create_readonly_text(
            value=self.band, name=AcceptSourceDialog.BAND)

        self.observatory_code_label = wx.StaticText(
            self, label=AcceptSourceDialog.OBSERVATORY_CODE)
        self.observatory_code_text = wx.TextCtrl(
            self, value=self.default_observatory_code,
            name=AcceptSourceDialog.OBSERVATORY_CODE)

    def _create_readonly_text(self, value, name):
        text_ctrl = wx.TextCtrl(self, style=wx.TE_READONLY, value=value,
                                name=name)
        text_ctrl.SetBackgroundColour(self.GetBackgroundColour())
        return text_ctrl

    def _get_vertical_widget_list(self):
        data_fields = [(self.minor_planet_num_label, self.minor_planet_num_text),
                       (self.provisional_name_label, self.provision_name_text),
                       (self.note1_label, self.note1_combobox),
                       (self.note2_label, self.note2_combobox),
                       (self.date_of_obs_label, self.date_of_obs_text),
                       (self.ra_label, self.ra_text),
                       (self.dec_label, self.dec_text),
                       (self.obs_mag_label, self.obs_mag_text),
                       (self.band_label, self.band_text),
                       (self.observatory_code_label, self.observatory_code_text),
        ]

        sizer = wx.FlexGridSizer(rows=len(data_fields), cols=2)
        for label, value in data_fields:
            sizer.Add(label)
            sizer.Add(value, proportion=1, flag=wx.EXPAND)

        sizer.AddGrowableCol(1, proportion=1)

        return [sizer]

    def _on_submit(self, event):
        # Grab data out of the form
        # TODO validation
        minor_planet_number = self.minor_planet_num_text.GetValue()
        note1 = self.note1_combobox.GetValue()
        note2 = self.note2_combobox.GetValue()
        obs_mag = self.obs_mag if not self.phot_failure else ""
        obs_mag_err = self.obs_mag_err if not self.phot_failure else -1
        band = self.band if not self.phot_failure else ""
        observatory_code = self.observatory_code_text.GetValue()
        comment = self.comment_text.GetValue()

        self.controller.on_do_accept(minor_planet_number,
                                     self.provisional_name,
                                     note1,
                                     note2,
                                     self.date_of_obs,
                                     self.ra,
                                     self.dec,
                                     obs_mag,
                                     obs_mag_err,
                                     band,
                                     observatory_code,
                                     comment)

    def _on_cancel(self, event):
        self.controller.on_cancel_accept()


class OffsetSourceDialog(SourceValidationDialog):
    TITLE = "Accept Re-centroid Dialog"

    def __init__(self, parent, controller, pix_coords=(0,0), cen_coords=(0,0)):

        self.cen_coords = cen_coords
        self.pix_coords = pix_coords
        self.default_comment =  "DAOphot centroid differs from input value.\n\n"
        self.default_comment += "{:8s} {:6.2f} {:6.2f}\n".format("mark", self.pix_coords[0], self.pix_coords[1])
        self.default_comment += "{:8s} {:6.2f} {:6.2f}\n".format("daophot", self.cen_coords[0], self.cen_coords[1])
        self.default_comment += "\nAccepted DAOphot centroid or Mark centroid?"

        super(OffsetSourceDialog, self).__init__(parent, title=self.TITLE)
        self.controller = controller

        self.submit_button.SetLabel("DAOPhot")
        self.cancel_button.SetLabel("Marker")

    def _init_ui(self):
        pass

    def _get_vertical_widget_list(self):
        return []

    def _on_submit(self, event):
        self.controller.on_do_offset(self.cen_coords)

    def _on_cancel(self, event):
        self.controller.on_cancel_offset(self.pix_coords)


class RejectSourceDialog(SourceValidationDialog):
    TITLE = "Reject Source"

    def __init__(self, parent, controller, default_comment=""):
        self.default_comment = default_comment

        super(RejectSourceDialog, self).__init__(parent, title=self.TITLE)
        self.controller = controller
        self.comment_text.SetFocus()

    def _init_ui(self):
        pass

    def _get_vertical_widget_list(self):
        return []

    def _on_submit(self, event):
        comment = self.comment_text.GetValue()
        self.controller.on_do_reject(comment)

    def _on_cancel(self, event):
        self.controller.on_cancel_reject()


class KeyboardCompleteComboBox(wx.ComboBox):
    """
    A combo-box with read-only, preset values.  When the user types a key
    in the text field, it will look through the possible choices and if
    there is one starting with that character, it will select it.
    """

    def __init__(self, parent, choices=None, **kwargs):
        if choices is None:
            choices = []

        self.choices = choices
        super(KeyboardCompleteComboBox, self).__init__(parent,
                                                       choices=choices,
                                                       style=wx.CB_READONLY,
                                                       **kwargs)
        self.Bind(wx.EVT_CHAR, self._on_char)

    def _on_char(self, event):
        keycode = event.GetKeyCode()

        if keycode == wx.WXK_RETURN:
            event.Skip()
            return

        if keycode < 0 or keycode > 255:
            # Something like a up arrow key, etc.  Ignore it.
            return

        char = chr(keycode)
        for choice in self.choices:
            if choice.startswith(char):
                self.SetStringSelection(choice)



"""
General-purpose list control widgets.
"""

__author__ = "David Rusk <drusk@uvic.ca>"

import wx
from wx.lib.mixins import listctrl as listmix


class ListCtrlAutoWidth(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent, size=wx.DefaultSize, style=0):
        super(ListCtrlAutoWidth, self).__init__(parent, size=size, style=style)
        listmix.ListCtrlAutoWidthMixin.__init__(self)


class ListCtrlPanel(wx.Panel, listmix.ColumnSorterMixin):
    """
    A list control panel with sortable columns.
    """

    def __init__(self, parent, columns):
        super(ListCtrlPanel, self).__init__(parent)

        self.list = ListCtrlAutoWidth(self,
                                      style=wx.LC_REPORT |
                                            wx.BORDER_NONE |
                                            wx.LC_EDIT_LABELS |
                                            wx.LC_SORT_ASCENDING
        )

        for i, column in enumerate(columns):
            self.list.InsertColumn(i, column)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.list, 1, wx.EXPAND)
        self.SetSizer(sizer)

    def populate_list(self, data):
        # Clear any existing data
        self.list.DeleteAllItems()

        item_data_map = {}
        index = 0
        for record in data:
            self.list.InsertStringItem(index, str(index))
            for colindex, item in enumerate(record):
                self.list.SetStringItem(index, colindex, str(item))

            # For ColumnSorterMixin
            self.list.SetItemData(index, index)
            item_data_map[index] = record

            index += 1

        self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)

        # This field is required to be set for the ColumnSorterMixin to work
        self.itemDataMap = item_data_map
        listmix.ColumnSorterMixin.__init__(self, 2)

    def GetListCtrl(self):
        """Required by ColumnSorterMixin"""
        return self.list


from __future__ import absolute_import

__author__ = 'jjk'
from unittest import TestCase
from ossos import mpc
from astropy.time import Time


class MPCParserTest(TestCase):
    """Did this MPC line parse correctly."""

    def test_standard_mpc_line(self):

        mpc_line = "     HL7j2    C2013 04 03.62926 17 12 01.16 +04 13 33.3          24.1 R      568"

        o = mpc.Observation.from_string(mpc_line)
        self.assertEqual(o.provisional_name, 'HL7j2')

    def test_long_provisional_mpc_line(self):

        mpc_line = "  XXXHL7j2    C2013 04 03.62926 17 12 01.16 +04 13 33.3          24.1 R      568"

        o = mpc.Observation.from_string(mpc_line)
        self.assertEqual(o.provisional_name, 'XXXHL7j2')

    def test_long_provisional_mpc_line_fail(self):
        '''What happens if there are too few spaces'''

        mpc_line = "XXXHL7j2      C2013 04 03.62926 17 12 01.16 +04 13 33.3          24.1 R      568"

        o = mpc.Observation.from_string(mpc_line)
        self.assertNotEqual(o.provisional_name, 'XXXHL7j2')

    def test_mpc_high_precision(self):
        """can we get the precision on the RA correct?"""

        mpc_line = " XXHL7j2      C2013 04 03.62926 17 12 01.161+04 13 33.3          24.1 R      568"

        o = mpc.Observation.from_string(mpc_line)
        print str(o.ra)
        self.assertEqual(str(o.ra), "17 12 01.161")

    def __test_ossos_format_mpc_line(self):

        mpc_lines_2=("     DRY001U   2013 04 09.36658 14 14 12.656-12 49 42.67         -1   r      568 None DRY001U Z  XXXXXX YYYYYY   UUUU % ",
                     "     DRY001U   2013 04 09.40970 14 14 11.877-12 49 37.60         -1   r      568 None DRY001U Z  XXXXXX YYYYYY   UUUU % ",
                     "     DRY001U   2013 04 09.45271 14 14 11.053-12 49 32.04         -1   r      568 None DRY001U Z  XXXXXX YYYYYY   UUUU % ")

        observations = []
        for line in mpc_lines_2:
            observations.append(mpc.Observation.from_string(line))
        name = observations[0].provisional_name

        query = ssos.Query(observations,
                           search_start_date=Time('2013-02-08', scale='utc'),
                           search_end_date=Time('2013-09-01', scale='utc'))

        ssos_result = query.get()

        ssos_data = ssos.SSOSParser(name).parse(ssos_result)

        print ssos_data.get_reading_count()



if __name__ == '__main__':
    unittest.main()
